@@ -8,20 +8,23 @@
 #include "box2d/math_functions.h"
 
 #include <stdarg.h>
+#include <stddef.h>
 #include <stdio.h>
 #include <vector>
 
-#if defined( _WIN32 )
-	#define _CRTDBG_MAP_ALLOC
-	#include <crtdbg.h>
-	#include <stdlib.h>
+#if defined( _MSC_VER )
+#define _CRTDBG_MAP_ALLOC
+#include <crtdbg.h>
+#include <stdlib.h>
 #else
-	#include <stdlib.h>
+#include <stdlib.h>
 #endif
 
+// clang-format off
 #include <glad/glad.h>
-// Keep glad.h before glfw3.h
 #include <GLFW/glfw3.h>
+// clang-format on
+
 #include <imgui.h>
 
 #define BUFFER_OFFSET( x ) ( (const void*)( x ) )
@@ -33,7 +36,7 @@ struct RGBA8
 	uint8_t r, g, b, a;
 };
 
-static inline RGBA8 MakeRGBA8( b2HexColor c, float alpha )
+static RGBA8 MakeRGBA8( b2HexColor c, float alpha )
 {
 	return { uint8_t( ( c >> 16 ) & 0xFF ), uint8_t( ( c >> 8 ) & 0xFF ), uint8_t( c & 0xFF ), uint8_t( 0xFF * alpha ) };
 }
@@ -154,7 +157,7 @@ struct GLBackground
 		glBufferData( GL_ARRAY_BUFFER, sizeof( vertices ), vertices, GL_STATIC_DRAW );
 		glVertexAttribPointer( vertexAttribute, 2, GL_FLOAT, GL_FALSE, 0, BUFFER_OFFSET( 0 ) );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -183,6 +186,8 @@ struct GLBackground
 		glUseProgram( m_programId );
 
 		float time = (float)glfwGetTime();
+		time = fmodf( time, 100.0f );
+
 		glUniform1f( m_timeUniform, time );
 		glUniform2f( m_resolutionUniform, (float)g_camera.m_width, (float)g_camera.m_height );
 
@@ -228,7 +233,7 @@ struct GLPoints
 						 "{\n"
 						 "	f_color = v_color;\n"
 						 "	gl_Position = projectionMatrix * vec4(v_position, 0.0f, 1.0f);\n"
-						 "   gl_PointSize = v_size;\n"
+						 "	gl_PointSize = v_size;\n"
 						 "}\n";
 
 		const char* fs = "#version 330\n"
@@ -256,16 +261,16 @@ struct GLPoints
 
 		// Vertex buffer
 		glBindBuffer( GL_ARRAY_BUFFER, m_vboId );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( PointData ), NULL, GL_DYNAMIC_DRAW );
+		glBufferData( GL_ARRAY_BUFFER, e_batchSize * sizeof( PointData ), nullptr, GL_DYNAMIC_DRAW );
 
 		glVertexAttribPointer( vertexAttribute, 2, GL_FLOAT, GL_FALSE, sizeof( PointData ),
 							   (void*)offsetof( PointData, position ) );
 		glVertexAttribPointer( sizeAttribute, 1, GL_FLOAT, GL_FALSE, sizeof( PointData ), (void*)offsetof( PointData, size ) );
-		// color will get automatically expanded to floats in the shader
+		// save bandwidth by expanding color to floats in the shader
 		glVertexAttribPointer( colorAttribute, 4, GL_UNSIGNED_BYTE, GL_TRUE, sizeof( PointData ),
 							   (void*)offsetof( PointData, rgba ) );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -320,14 +325,14 @@ struct GLPoints
 		int base = 0;
 		while ( count > 0 )
 		{
-			int batchCount = b2MinInt( count, e_maxCount );
+			int batchCount = b2MinInt( count, e_batchSize );
 			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( PointData ), &m_points[base] );
 			glDrawArrays( GL_POINTS, 0, batchCount );
 
-			CheckErrorGL();
+			CheckOpenGL();
 
-			count -= e_maxCount;
-			base += e_maxCount;
+			count -= e_batchSize;
+			base += e_batchSize;
 		}
 
 		glDisable( GL_PROGRAM_POINT_SIZE );
@@ -340,7 +345,7 @@ struct GLPoints
 
 	enum
 	{
-		e_maxCount = 2048
+		e_batchSize = 2048
 	};
 
 	std::vector<PointData> m_points;
@@ -361,24 +366,30 @@ struct GLLines
 {
 	void Create()
 	{
-		const char* vs = "#version 330\n"
-						 "uniform mat4 projectionMatrix;\n"
-						 "layout(location = 0) in vec2 v_position;\n"
-						 "layout(location = 1) in vec4 v_color;\n"
-						 "out vec4 f_color;\n"
-						 "void main(void)\n"
-						 "{\n"
-						 "	f_color = v_color;\n"
-						 "	gl_Position = projectionMatrix * vec4(v_position, 0.0f, 1.0f);\n"
-						 "}\n";
+		const char* vs =
+			R"(
+			#version 330
+			uniform mat4 projectionMatrix;
+			layout(location = 0) in vec2 v_position;
+			layout(location = 1) in vec4 v_color;
+			out vec4 f_color;
+			void main(void)
+			{
+				f_color = v_color;
+				gl_Position = projectionMatrix * vec4(v_position, 0.0f, 1.0f);
+			}
+			)";
 
-		const char* fs = "#version 330\n"
-						 "in vec4 f_color;\n"
-						 "out vec4 color;\n"
-						 "void main(void)\n"
-						 "{\n"
-						 "	color = f_color;\n"
-						 "}\n";
+		const char* fs =
+			R"(
+			#version 330
+			in vec4 f_color;
+			out vec4 color;
+			void main(void)
+			{
+				color = f_color;
+			}
+			)";
 
 		m_programId = CreateProgramFromStrings( vs, fs );
 		m_projectionUniform = glGetUniformLocation( m_programId, "projectionMatrix" );
@@ -395,15 +406,15 @@ struct GLLines
 
 		// Vertex buffer
 		glBindBuffer( GL_ARRAY_BUFFER, m_vboId );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( VertexData ), NULL, GL_DYNAMIC_DRAW );
+		glBufferData( GL_ARRAY_BUFFER, e_batchSize * sizeof( VertexData ), nullptr, GL_DYNAMIC_DRAW );
 
 		glVertexAttribPointer( vertexAttribute, 2, GL_FLOAT, GL_FALSE, sizeof( VertexData ),
 							   (void*)offsetof( VertexData, position ) );
-		// color will get automatically expanded to floats in the shader
+		// save bandwidth by expanding color to floats in the shader
 		glVertexAttribPointer( colorAttribute, 4, GL_UNSIGNED_BYTE, GL_TRUE, sizeof( VertexData ),
 							   (void*)offsetof( VertexData, rgba ) );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -444,6 +455,11 @@ struct GLLines
 
 		assert( count % 2 == 0 );
 
+		glEnable( GL_LINE_SMOOTH );
+		glEnable( GL_BLEND );
+		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+		glLineWidth( 1.0f );
+
 		glUseProgram( m_programId );
 
 		float proj[16] = { 0.0f };
@@ -458,21 +474,23 @@ struct GLLines
 		int base = 0;
 		while ( count > 0 )
 		{
-			int batchCount = b2MinInt( count, e_maxCount );
+			int batchCount = b2MinInt( count, e_batchSize );
 			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( VertexData ), &m_points[base] );
 
 			glDrawArrays( GL_LINES, 0, batchCount );
 
-			CheckErrorGL();
+			CheckOpenGL();
 
-			count -= e_maxCount;
-			base += e_maxCount;
+			count -= e_batchSize;
+			base += e_batchSize;
 		}
 
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
 		glBindVertexArray( 0 );
 		glUseProgram( 0 );
 
+		glDisable( GL_BLEND );
+
 		m_points.clear();
 	}
 
@@ -481,7 +499,7 @@ struct GLLines
 	enum
 	{
 		// must be multiple of 2
-		e_maxCount = 2 * 2048
+		e_batchSize = 2 * 2048
 	};
 
 	std::vector<VertexData> m_points;
@@ -492,150 +510,6 @@ struct GLLines
 	GLint m_projectionUniform;
 };
 
-// todo this is not used anymore and has untested changes
-struct GLTriangles
-{
-	void Create()
-	{
-		const char* vs = "#version 330\n"
-						 "uniform mat4 projectionMatrix;\n"
-						 "layout(location = 0) in vec2 v_position;\n"
-						 "layout(location = 1) in vec4 v_color;\n"
-						 "out vec4 f_color;\n"
-						 "void main(void)\n"
-						 "{\n"
-						 "	f_color = v_color;\n"
-						 "	gl_Position = projectionMatrix * vec4(v_position, 0.0f, 1.0f);\n"
-						 "}\n";
-
-		const char* fs = "#version 330\n"
-						 "in vec4 f_color;\n"
-						 "out vec4 color;\n"
-						 "void main(void)\n"
-						 "{\n"
-						 "	color = f_color;\n"
-						 "}\n";
-
-		m_programId = CreateProgramFromStrings( vs, fs );
-		m_projectionUniform = glGetUniformLocation( m_programId, "projectionMatrix" );
-		int vertexAttribute = 0;
-		int colorAttribute = 1;
-
-		// Generate
-		glGenVertexArrays( 1, &m_vaoId );
-		glGenBuffers( 1, &m_vboId );
-
-		glBindVertexArray( m_vaoId );
-		glEnableVertexAttribArray( vertexAttribute );
-		glEnableVertexAttribArray( colorAttribute );
-
-		// Vertex buffer
-		glBindBuffer( GL_ARRAY_BUFFER, m_vboId );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( VertexData ), NULL, GL_DYNAMIC_DRAW );
-
-		glVertexAttribPointer( vertexAttribute, 2, GL_FLOAT, GL_FALSE, sizeof( VertexData ),
-							   (void*)offsetof( VertexData, position ) );
-		// color will get automatically expanded to floats in the shader
-		glVertexAttribPointer( colorAttribute, 4, GL_UNSIGNED_BYTE, GL_TRUE, sizeof( VertexData ),
-							   (void*)offsetof( VertexData, rgba ) );
-
-		CheckErrorGL();
-
-		// Cleanup
-		glBindBuffer( GL_ARRAY_BUFFER, 0 );
-		glBindVertexArray( 0 );
-	}
-
-	void Destroy()
-	{
-		if ( m_vaoId )
-		{
-			glDeleteVertexArrays( 1, &m_vaoId );
-			glDeleteBuffers( 1, &m_vboId );
-			m_vaoId = 0;
-			m_vboId = 0;
-		}
-
-		if ( m_programId )
-		{
-			glDeleteProgram( m_programId );
-			m_programId = 0;
-		}
-	}
-
-	void AddTriangle( b2Vec2 p1, b2Vec2 p2, b2Vec2 p3, b2HexColor c )
-	{
-		RGBA8 rgba = MakeRGBA8( c, 1.0f );
-		m_points.push_back( { p1, rgba } );
-		m_points.push_back( { p2, rgba } );
-		m_points.push_back( { p3, rgba } );
-	}
-
-	void Flush()
-	{
-		int count = (int)m_points.size();
-		if ( count == 0 )
-		{
-			return;
-		}
-
-		assert( count % 3 == 0 );
-
-		glUseProgram( m_programId );
-
-		float proj[16] = { 0.0f };
-		g_camera.BuildProjectionMatrix( proj, 0.2f );
-
-		glUniformMatrix4fv( m_projectionUniform, 1, GL_FALSE, proj );
-
-		glBindVertexArray( m_vaoId );
-
-		glBindBuffer( GL_ARRAY_BUFFER, m_vboId );
-		glEnable( GL_BLEND );
-		glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
-
-		int base = 0;
-		while ( count > 0 )
-		{
-			int batchCount = b2MinInt( count, e_maxCount );
-
-			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( VertexData ), &m_points[base] );
-			glDrawArrays( GL_TRIANGLES, 0, batchCount );
-
-			CheckErrorGL();
-
-			count -= e_maxCount;
-			base += e_maxCount;
-		}
-
-		glDisable( GL_BLEND );
-
-		glBindBuffer( GL_ARRAY_BUFFER, 0 );
-		glBindVertexArray( 0 );
-		glUseProgram( 0 );
-
-		m_points.clear();
-	}
-
-	enum
-	{
-		// must be multiple of 3
-		e_maxCount = 3 * 512
-	};
-
-	std::vector<VertexData> m_points;
-
-	GLuint m_vaoId;
-	GLuint m_vboId;
-	GLuint m_programId;
-	GLint m_projectionUniform;
-};
-
-struct Transform
-{
-	float x, y, c, s;
-};
-
 struct CircleData
 {
 	b2Vec2 position;
@@ -674,7 +548,7 @@ struct GLCircles
 
 		// Circle buffer
 		glBindBuffer( GL_ARRAY_BUFFER, m_vboIds[1] );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( CircleData ), NULL, GL_DYNAMIC_DRAW );
+		glBufferData( GL_ARRAY_BUFFER, e_batchSize * sizeof( CircleData ), nullptr, GL_DYNAMIC_DRAW );
 
 		glVertexAttribPointer( positionInstance, 2, GL_FLOAT, GL_FALSE, sizeof( CircleData ),
 							   (void*)offsetof( CircleData, position ) );
@@ -687,7 +561,7 @@ struct GLCircles
 		glVertexAttribDivisor( radiusInstance, 1 );
 		glVertexAttribDivisor( colorInstance, 1 );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -743,15 +617,15 @@ struct GLCircles
 		int base = 0;
 		while ( count > 0 )
 		{
-			int batchCount = b2MinInt( count, e_maxCount );
+			int batchCount = b2MinInt( count, e_batchSize );
 
 			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( CircleData ), &m_circles[base] );
 			glDrawArraysInstanced( GL_TRIANGLES, 0, 6, batchCount );
 
-			CheckErrorGL();
+			CheckOpenGL();
 
-			count -= e_maxCount;
-			base += e_maxCount;
+			count -= e_batchSize;
+			base += e_batchSize;
 		}
 
 		glDisable( GL_BLEND );
@@ -765,7 +639,7 @@ struct GLCircles
 
 	enum
 	{
-		e_maxCount = 2048
+		e_batchSize = 2048
 	};
 
 	std::vector<CircleData> m_circles;
@@ -779,7 +653,7 @@ struct GLCircles
 
 struct SolidCircleData
 {
-	Transform transform;
+	b2Transform transform;
 	float radius;
 	RGBA8 rgba;
 };
@@ -819,7 +693,7 @@ struct GLSolidCircles
 
 		// Circle buffer
 		glBindBuffer( GL_ARRAY_BUFFER, m_vboIds[1] );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( SolidCircleData ), NULL, GL_DYNAMIC_DRAW );
+		glBufferData( GL_ARRAY_BUFFER, e_batchSize * sizeof( SolidCircleData ), nullptr, GL_DYNAMIC_DRAW );
 
 		glVertexAttribPointer( transformInstance, 4, GL_FLOAT, GL_FALSE, sizeof( SolidCircleData ),
 							   (void*)offsetof( SolidCircleData, transform ) );
@@ -832,7 +706,7 @@ struct GLSolidCircles
 		glVertexAttribDivisor( radiusInstance, 1 );
 		glVertexAttribDivisor( colorInstance, 1 );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -860,7 +734,7 @@ struct GLSolidCircles
 	void AddCircle( const b2Transform& transform, float radius, b2HexColor color )
 	{
 		RGBA8 rgba = MakeRGBA8( color, 1.0f );
-		m_circles.push_back( { { transform.p.x, transform.p.y, transform.q.c, transform.q.s }, radius, rgba } );
+		m_circles.push_back( { transform, radius, rgba } );
 	}
 
 	void Flush()
@@ -888,15 +762,15 @@ struct GLSolidCircles
 		int base = 0;
 		while ( count > 0 )
 		{
-			int batchCount = b2MinInt( count, e_maxCount );
+			int batchCount = b2MinInt( count, e_batchSize );
 
 			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( SolidCircleData ), &m_circles[base] );
 			glDrawArraysInstanced( GL_TRIANGLES, 0, 6, batchCount );
 
-			CheckErrorGL();
+			CheckOpenGL();
 
-			count -= e_maxCount;
-			base += e_maxCount;
+			count -= e_batchSize;
+			base += e_batchSize;
 		}
 
 		glDisable( GL_BLEND );
@@ -910,7 +784,7 @@ struct GLSolidCircles
 
 	enum
 	{
-		e_maxCount = 2048
+		e_batchSize = 2048
 	};
 
 	std::vector<SolidCircleData> m_circles;
@@ -924,7 +798,7 @@ struct GLSolidCircles
 
 struct CapsuleData
 {
-	Transform transform;
+	b2Transform transform;
 	float radius;
 	float length;
 	RGBA8 rgba;
@@ -966,7 +840,7 @@ struct GLSolidCapsules
 
 		// Capsule buffer
 		glBindBuffer( GL_ARRAY_BUFFER, m_vboIds[1] );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( CapsuleData ), NULL, GL_DYNAMIC_DRAW );
+		glBufferData( GL_ARRAY_BUFFER, e_batchSize * sizeof( CapsuleData ), nullptr, GL_DYNAMIC_DRAW );
 
 		glVertexAttribPointer( transformInstance, 4, GL_FLOAT, GL_FALSE, sizeof( CapsuleData ),
 							   (void*)offsetof( CapsuleData, transform ) );
@@ -982,7 +856,7 @@ struct GLSolidCapsules
 		glVertexAttribDivisor( lengthInstance, 1 );
 		glVertexAttribDivisor( colorInstance, 1 );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -1025,7 +899,7 @@ struct GLSolidCapsules
 
 		RGBA8 rgba = MakeRGBA8( c, 1.0f );
 
-		m_capsules.push_back( { { transform.p.x, transform.p.y, transform.q.c, transform.q.s }, radius, length, rgba } );
+		m_capsules.push_back( { transform, radius, length, rgba } );
 	}
 
 	void Flush()
@@ -1053,15 +927,15 @@ struct GLSolidCapsules
 		int base = 0;
 		while ( count > 0 )
 		{
-			int batchCount = b2MinInt( count, e_maxCount );
+			int batchCount = b2MinInt( count, e_batchSize );
 
 			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( CapsuleData ), &m_capsules[base] );
 			glDrawArraysInstanced( GL_TRIANGLES, 0, 6, batchCount );
 
-			CheckErrorGL();
+			CheckOpenGL();
 
-			count -= e_maxCount;
-			base += e_maxCount;
+			count -= e_batchSize;
+			base += e_batchSize;
 		}
 
 		glDisable( GL_BLEND );
@@ -1075,7 +949,7 @@ struct GLSolidCapsules
 
 	enum
 	{
-		e_maxCount = 2048
+		e_batchSize = 2048
 	};
 
 	std::vector<CapsuleData> m_capsules;
@@ -1141,7 +1015,7 @@ struct GLSolidPolygons
 
 		// Polygon buffer
 		glBindBuffer( GL_ARRAY_BUFFER, m_vboIds[1] );
-		glBufferData( GL_ARRAY_BUFFER, e_maxCount * sizeof( PolygonData ), NULL, GL_DYNAMIC_DRAW );
+		glBufferData( GL_ARRAY_BUFFER, e_batchSize * sizeof( PolygonData ), nullptr, GL_DYNAMIC_DRAW );
 		glVertexAttribPointer( instanceTransform, 4, GL_FLOAT, GL_FALSE, sizeof( PolygonData ),
 							   (void*)offsetof( PolygonData, transform ) );
 		glVertexAttribPointer( instancePoint12, 4, GL_FLOAT, GL_FALSE, sizeof( PolygonData ),
@@ -1169,7 +1043,7 @@ struct GLSolidPolygons
 		glVertexAttribDivisor( instanceRadius, 1 );
 		glVertexAttribDivisor( instanceColor, 1 );
 
-		CheckErrorGL();
+		CheckOpenGL();
 
 		// Cleanup
 		glBindBuffer( GL_ARRAY_BUFFER, 0 );
@@ -1236,14 +1110,14 @@ struct GLSolidPolygons
 		int base = 0;
 		while ( count > 0 )
 		{
-			int batchCount = b2MinInt( count, e_maxCount );
+			int batchCount = b2MinInt( count, e_batchSize );
 
 			glBufferSubData( GL_ARRAY_BUFFER, 0, batchCount * sizeof( PolygonData ), &m_polygons[base] );
 			glDrawArraysInstanced( GL_TRIANGLES, 0, 6, batchCount );
-			CheckErrorGL();
+			CheckOpenGL();
 
-			count -= e_maxCount;
-			base += e_maxCount;
+			count -= e_batchSize;
+			base += e_batchSize;
 		}
 
 		glDisable( GL_BLEND );
@@ -1257,7 +1131,7 @@ struct GLSolidPolygons
 
 	enum
 	{
-		e_maxCount = 512
+		e_batchSize = 512
 	};
 
 	std::vector<PolygonData> m_polygons;
@@ -1290,11 +1164,6 @@ void DrawSolidCircleFcn( b2Transform transform, float radius, b2HexColor color,
 	static_cast<Draw*>( context )->DrawSolidCircle( transform, b2Vec2_zero, radius, color );
 }
 
-void DrawCapsuleFcn( b2Vec2 p1, b2Vec2 p2, float radius, b2HexColor color, void* context )
-{
-	static_cast<Draw*>( context )->DrawCapsule( p1, p2, radius, color );
-}
-
 void DrawSolidCapsuleFcn( b2Vec2 p1, b2Vec2 p2, float radius, b2HexColor color, void* context )
 {
 	static_cast<Draw*>( context )->DrawSolidCapsule( p1, p2, radius, color );
@@ -1315,7 +1184,7 @@ void DrawPointFcn( b2Vec2 p, float size, b2HexColor color, void* context )
 	static_cast<Draw*>( context )->DrawPoint( p, size, color );
 }
 
-void DrawStringFcn( b2Vec2 p, const char* s, void* context )
+void DrawStringFcn( b2Vec2 p, const char* s, b2HexColor color, void* context )
 {
 	static_cast<Draw*>( context )->DrawString( p, s );
 }
@@ -1325,23 +1194,27 @@ Draw::Draw()
 	m_showUI = true;
 	m_points = nullptr;
 	m_lines = nullptr;
-	m_triangles = nullptr;
 	m_circles = nullptr;
 	m_solidCircles = nullptr;
 	m_solidCapsules = nullptr;
 	m_solidPolygons = nullptr;
 	m_debugDraw = {};
+	m_smallFont = nullptr;
+	m_mediumFont = nullptr;
+	m_largeFont = nullptr;
+	m_regularFont = nullptr;
+	m_background = nullptr;
 }
 
 Draw::~Draw()
 {
 	assert( m_points == nullptr );
 	assert( m_lines == nullptr );
-	assert( m_triangles == nullptr );
 	assert( m_circles == nullptr );
 	assert( m_solidCircles == nullptr );
 	assert( m_solidCapsules == nullptr );
 	assert( m_solidPolygons == nullptr );
+	assert( m_background == nullptr );
 }
 
 void Draw::Create()
@@ -1352,8 +1225,6 @@ void Draw::Create()
 	m_points->Create();
 	m_lines = new GLLines;
 	m_lines->Create();
-	m_triangles = new GLTriangles;
-	m_triangles->Create();
 	m_circles = new GLCircles;
 	m_circles->Create();
 	m_solidCircles = new GLSolidCircles;
@@ -1365,29 +1236,34 @@ void Draw::Create()
 
 	b2AABB bounds = { { -FLT_MAX, -FLT_MAX }, { FLT_MAX, FLT_MAX } };
 
-	m_debugDraw = { DrawPolygonFcn,
-					DrawSolidPolygonFcn,
-					DrawCircleFcn,
-					DrawSolidCircleFcn,
-					DrawCapsuleFcn,
-					DrawSolidCapsuleFcn,
-					DrawSegmentFcn,
-					DrawTransformFcn,
-					DrawPointFcn,
-					DrawStringFcn,
-					bounds,
-					false, // drawUsingBounds
-					true,  // shapes
-					true,  // joints
-					false, // joint extras
-					false, // aabbs
-					false, // mass
-					false, // contacts
-					false, // colors
-					false, // normals
-					false, // impulse
-					false, // friction
-					this };
+	m_debugDraw = {};
+
+	m_debugDraw.DrawPolygonFcn = DrawPolygonFcn;
+	m_debugDraw.DrawSolidPolygonFcn = DrawSolidPolygonFcn;
+	m_debugDraw.DrawCircleFcn = DrawCircleFcn;
+	m_debugDraw.DrawSolidCircleFcn = DrawSolidCircleFcn;
+	m_debugDraw.DrawSolidCapsuleFcn = DrawSolidCapsuleFcn;
+	m_debugDraw.DrawSegmentFcn = DrawSegmentFcn;
+	m_debugDraw.DrawTransformFcn = DrawTransformFcn;
+	m_debugDraw.DrawPointFcn = DrawPointFcn;
+	m_debugDraw.DrawStringFcn = DrawStringFcn;
+	m_debugDraw.drawingBounds = bounds;
+
+	m_debugDraw.useDrawingBounds = false;
+	m_debugDraw.drawShapes = true;
+	m_debugDraw.drawJoints = true;
+	m_debugDraw.drawJointExtras = false;
+	m_debugDraw.drawBounds = false;
+	m_debugDraw.drawMass = false;
+	m_debugDraw.drawContacts = false;
+	m_debugDraw.drawGraphColors = false;
+	m_debugDraw.drawContactNormals = false;
+	m_debugDraw.drawContactImpulses = false;
+	m_debugDraw.drawContactFeatures = false;
+	m_debugDraw.drawFrictionImpulses = false;
+	m_debugDraw.drawIslands = false;
+
+	m_debugDraw.context = this;
 }
 
 void Draw::Destroy()
@@ -1404,10 +1280,6 @@ void Draw::Destroy()
 	delete m_lines;
 	m_lines = nullptr;
 
-	m_triangles->Destroy();
-	delete m_triangles;
-	m_triangles = nullptr;
-
 	m_circles->Destroy();
 	delete m_circles;
 	m_circles = nullptr;
@@ -1452,59 +1324,6 @@ void Draw::DrawSolidCircle( b2Transform transform, b2Vec2 center, float radius,
 	m_solidCircles->AddCircle( transform, radius, color );
 }
 
-// todo this is not used
-void Draw::DrawCapsule( b2Vec2 p1, b2Vec2 p2, float radius, b2HexColor color )
-{
-	float length;
-	b2Vec2 axis = b2GetLengthAndNormalize( &length, b2Sub( p2, p1 ) );
-
-	if ( length == 0.0f )
-	{
-		DrawCircle( p1, radius, color );
-	}
-
-	const float k_segments = 16.0f;
-	const float k_increment = b2_pi / k_segments;
-	float sinInc = sinf( k_increment );
-	float cosInc = cosf( k_increment );
-
-	b2Vec2 r1 = { -axis.y, axis.x };
-	b2Vec2 v1 = b2MulAdd( p1, radius, r1 );
-	b2Vec2 a = v1;
-	for ( int i = 0; i < k_segments; ++i )
-	{
-		// Perform rotation to avoid additional trigonometry.
-		b2Vec2 r2;
-		r2.x = cosInc * r1.x - sinInc * r1.y;
-		r2.y = sinInc * r1.x + cosInc * r1.y;
-		b2Vec2 v2 = b2MulAdd( p1, radius, r2 );
-		m_lines->AddLine( v1, v2, color );
-		r1 = r2;
-		v1 = v2;
-	}
-	b2Vec2 b = v1;
-
-	r1 = { axis.y, -axis.x };
-	v1 = b2MulAdd( p2, radius, r1 );
-	b2Vec2 c = v1;
-	for ( int i = 0; i < k_segments; ++i )
-	{
-		// Perform rotation to avoid additional trigonometry.
-		b2Vec2 r2;
-		r2.x = cosInc * r1.x - sinInc * r1.y;
-		r2.y = sinInc * r1.x + cosInc * r1.y;
-		b2Vec2 v2 = b2MulAdd( p2, radius, r2 );
-		m_lines->AddLine( v1, v2, color );
-		r1 = r2;
-		v1 = v2;
-	}
-	b2Vec2 d = v1;
-
-	m_lines->AddLine( a, d, color );
-	m_lines->AddLine( b, c, color );
-	m_lines->AddLine( p1, p2, color );
-}
-
 void Draw::DrawSolidCapsule( b2Vec2 p1, b2Vec2 p2, float radius, b2HexColor color )
 {
 	m_solidCapsules->AddCapsule( p1, p2, radius, color );
@@ -1544,8 +1363,10 @@ void Draw::DrawString( int x, int y, const char* string, ... )
 	ImGui::Begin( "Overlay", nullptr,
 				  ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoInputs | ImGuiWindowFlags_AlwaysAutoResize |
 					  ImGuiWindowFlags_NoScrollbar );
+	ImGui::PushFont( g_draw.m_regularFont );
 	ImGui::SetCursorPos( ImVec2( float( x ), float( y ) ) );
 	ImGui::TextColoredV( ImColor( 230, 153, 153, 255 ), string, arg );
+	ImGui::PopFont();
 	ImGui::End();
 	va_end( arg );
 }
@@ -1583,11 +1404,10 @@ void Draw::Flush()
 	m_solidCircles->Flush();
 	m_solidCapsules->Flush();
 	m_solidPolygons->Flush();
-	m_triangles->Flush();
 	m_circles->Flush();
 	m_lines->Flush();
 	m_points->Flush();
-	CheckErrorGL();
+	CheckOpenGL();
 }
 
 void Draw::DrawBackground()
