@@ -1,10 +1,16 @@
 // SPDX-FileCopyrightText: 2023 Erin Catto
 // SPDX-License-Identifier: MIT
 
+#if defined( _MSC_VER )
+#ifndef _CRT_SECURE_NO_WARNINGS
 #define _CRT_SECURE_NO_WARNINGS
+#endif
 #define _CRTDBG_MAP_ALLOC
+#endif
+
 #define IMGUI_DISABLE_OBSOLETE_FUNCTIONS 1
 
+#include "TaskScheduler.h"
 #include "draw.h"
 #include "sample.h"
 #include "settings.h"
@@ -13,23 +19,26 @@
 #include "box2d/box2d.h"
 #include "box2d/math_functions.h"
 
-#include <glad/glad.h>
-// Keep glad.h before glfw3.h
-#include <GLFW/glfw3.h>
-#include <imgui.h>
-#include <imgui_impl_glfw.h>
-#include <imgui_impl_opengl3.h>
+// clang-format off
+#include "glad/glad.h"
+#include "GLFW/glfw3.h"
+// clang-format on
+
+#include "imgui.h"
+#include "imgui_impl_glfw.h"
+#include "imgui_impl_opengl3.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
 #ifdef BOX2D_PROFILE
-	#include <tracy/Tracy.hpp>
+#include <tracy/Tracy.hpp>
 #else
-	#define FrameMark
+#define FrameMark
 #endif
 
-#if defined( _WIN32 )
-	#include <crtdbg.h>
+#if defined( _MSC_VER ) && 0
+#include <crtdbg.h>
 
 static int MyAllocHook( int allocType, void* userData, size_t size, int blockType, long requestNumber,
 						const unsigned char* filename, int lineNumber )
@@ -66,7 +75,7 @@ void* AllocFcn( uint32_t size, int32_t alignment )
 	size_t sizeAligned = ( ( size - 1 ) | ( alignment - 1 ) ) + 1;
 	assert( ( sizeAligned & ( alignment - 1 ) ) == 0 );
 
-#if defined( _WIN64 )
+#if defined( _MSC_VER )
 	void* ptr = _aligned_malloc( sizeAligned, alignment );
 #else
 	void* ptr = aligned_alloc( alignment, sizeAligned );
@@ -77,7 +86,7 @@ void* AllocFcn( uint32_t size, int32_t alignment )
 
 void FreeFcn( void* mem )
 {
-#if defined( _WIN64 )
+#if defined( _MSC_VER )
 	_aligned_free( mem );
 #else
 	free( mem );
@@ -95,7 +104,7 @@ void glfwErrorCallback( int error, const char* description )
 	fprintf( stderr, "GLFW error occurred. Code: %d. Description: %s\n", error, description );
 }
 
-static inline int CompareSamples( const void* a, const void* b )
+static int CompareSamples( const void* a, const void* b )
 {
 	SampleEntry* sa = (SampleEntry*)a;
 	SampleEntry* sb = (SampleEntry*)b;
@@ -109,7 +118,7 @@ static inline int CompareSamples( const void* a, const void* b )
 	return result;
 }
 
-static void SortTests()
+static void SortSamples()
 {
 	qsort( g_sampleEntries, g_sampleCount, sizeof( SampleEntry ), CompareSamples );
 }
@@ -142,17 +151,17 @@ static void CreateUI( GLFWwindow* window, const char* glslVersion )
 		assert( false );
 	}
 
-	// this doesn't look that good
-	// Search for font file
 	const char* fontPath = "samples/data/droid_sans.ttf";
 	FILE* file = fopen( fontPath, "rb" );
 
-	if ( file != NULL )
+	if ( file != nullptr )
 	{
 		ImFontConfig fontConfig;
 		fontConfig.RasterizerMultiply = s_windowScale * s_framebufferScale;
-		ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 14.0f, &fontConfig );
-		ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 48.0f, &fontConfig );
+		g_draw.m_smallFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 14.0f, &fontConfig );
+		g_draw.m_regularFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 18.0f, &fontConfig );
+		g_draw.m_mediumFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 40.0f, &fontConfig );
+		g_draw.m_largeFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 64.0f, &fontConfig );
 	}
 	else
 	{
@@ -413,13 +422,16 @@ static void UpdateUI()
 				ImGui::Checkbox( "Shapes", &s_settings.drawShapes );
 				ImGui::Checkbox( "Joints", &s_settings.drawJoints );
 				ImGui::Checkbox( "Joint Extras", &s_settings.drawJointExtras );
-				ImGui::Checkbox( "AABBs", &s_settings.drawAABBs );
+				ImGui::Checkbox( "Bounds", &s_settings.drawBounds );
 				ImGui::Checkbox( "Contact Points", &s_settings.drawContactPoints );
 				ImGui::Checkbox( "Contact Normals", &s_settings.drawContactNormals );
 				ImGui::Checkbox( "Contact Impulses", &s_settings.drawContactImpulses );
+				ImGui::Checkbox( "Contact Features", &s_settings.drawContactFeatures );
 				ImGui::Checkbox( "Friction Impulses", &s_settings.drawFrictionImpulses );
-				ImGui::Checkbox( "Center of Masses", &s_settings.drawMass );
+				ImGui::Checkbox( "Mass", &s_settings.drawMass );
+				ImGui::Checkbox( "Body Names", &s_settings.drawBodyNames );
 				ImGui::Checkbox( "Graph Colors", &s_settings.drawGraphColors );
+				ImGui::Checkbox( "Islands", &s_settings.drawIslands );
 				ImGui::Checkbox( "Counters", &s_settings.drawCounters );
 				ImGui::Checkbox( "Profile", &s_settings.drawProfile );
 
@@ -462,7 +474,7 @@ static void UpdateUI()
 
 			ImGuiTreeNodeFlags nodeFlags = ImGuiTreeNodeFlags_OpenOnArrow | ImGuiTreeNodeFlags_OpenOnDoubleClick;
 
-			if ( ImGui::BeginTabItem( "Tests" ) )
+			if ( ImGui::BeginTabItem( "Samples" ) )
 			{
 				int categoryIndex = 0;
 				const char* category = g_sampleEntries[categoryIndex].category;
@@ -513,22 +525,21 @@ static void UpdateUI()
 
 		ImGui::End();
 
-		s_sample->UpdateUI();
+		s_sample->UpdateGui();
 	}
 }
 
-//
 int main( int, char** )
 {
-#if defined( _WIN32 )
+#if defined( _MSC_VER )
 	// Enable memory-leak reports
 	_CrtSetReportMode( _CRT_WARN, _CRTDBG_MODE_DEBUG | _CRTDBG_MODE_FILE );
 	_CrtSetReportFile( _CRT_WARN, _CRTDBG_FILE_STDOUT );
 	//_CrtSetAllocHook(MyAllocHook);
 
-	// How to break at the leaking allocation, in the watch window enter this variable
-	// and set it to the allocation number in {}. Do this at the first line in main.
-	// {,,ucrtbased.dll}_crtBreakAlloc = <allocation number> 3970
+// How to break at the leaking allocation, in the watch window enter this variable
+// and set it to the allocation number in {}. Do this at the first line in main.
+// {,,ucrtbased.dll}_crtBreakAlloc = <allocation number>
 #endif
 
 	// Install memory hooks
@@ -539,7 +550,8 @@ int main( int, char** )
 
 	s_settings.Load();
 	s_settings.workerCount = b2MinInt( 8, (int)enki::GetNumHardwareThreads() / 2 );
-	SortTests();
+
+	SortSamples();
 
 	glfwSetErrorCallback( glfwErrorCallback );
 
@@ -564,7 +576,7 @@ int main( int, char** )
 	glfwWindowHint( GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE );
 
 	// MSAA
-	glfwWindowHint( GLFW_SAMPLES, 4 );
+	//glfwWindowHint( GLFW_SAMPLES, 4 );
 
 	b2Version version = b2GetVersion();
 	snprintf( buffer, 128, "Box2D Version %d.%d.%d", version.major, version.minor, version.revision );
@@ -613,8 +625,19 @@ int main( int, char** )
 		return -1;
 	}
 
-	printf( "GL %d.%d\n", GLVersion.major, GLVersion.minor );
-	printf( "OpenGL %s, GLSL %s\n", glGetString( GL_VERSION ), glGetString( GL_SHADING_LANGUAGE_VERSION ) );
+	{
+		//float temp1[2] = {};
+		//glGetFloatv( GL_ALIASED_LINE_WIDTH_RANGE, temp1 );
+
+		//float temp2[2] = {};
+		//glGetFloatv( GL_SMOOTH_LINE_WIDTH_RANGE, temp2 );
+
+		const char* glVersionString = (const char*)glGetString( GL_VERSION );
+		const char* glslVersionString = (const char*)glGetString( GL_SHADING_LANGUAGE_VERSION );
+		printf( "OpenGL %s, GLSL %s\n", glVersionString, glslVersionString );
+		//printf( "OpenGL aliased line width range : [%g, %g]\n", temp1[0], temp1[1] );
+		//printf( "OpenGL smooth line width range : [%g, %g]\n", temp2[0], temp2[1] );
+	}
 
 	glfwSetWindowSizeCallback( g_mainWindow, ResizeWindowCallback );
 	glfwSetKeyCallback( g_mainWindow, KeyCallback );
@@ -634,8 +657,6 @@ int main( int, char** )
 
 	float frameTime = 0.0;
 
-	int32_t frame = 0;
-
 	while ( !glfwWindowShouldClose( g_mainWindow ) )
 	{
 		double time1 = glfwGetTime();
@@ -661,7 +682,7 @@ int main( int, char** )
 
 		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
 
-		g_draw.DrawBackground();
+		// g_draw.DrawBackground();
 
 		double cursorPosX = 0, cursorPosY = 0;
 		glfwGetCursorPos( g_mainWindow, &cursorPosX, &cursorPosY );
@@ -707,11 +728,11 @@ int main( int, char** )
 
 		// ImGui::ShowDemoWindow();
 
-		// if (g_draw.m_showUI)
+		if ( g_draw.m_showUI )
 		{
-			 snprintf( buffer, 128, "%.1f ms - step %d - camera (%g, %g, %g)", 1000.0f * frameTime, s_sample->m_stepCount,
+			snprintf( buffer, 128, "%.1f ms - step %d - camera (%g, %g, %g)", 1000.0f * frameTime, s_sample->m_stepCount,
 					  g_camera.m_center.x, g_camera.m_center.y, g_camera.m_zoom );
-			//snprintf( buffer, 128, "%.1f ms", 1000.0f * frameTime );
+			// snprintf( buffer, 128, "%.1f ms", 1000.0f * frameTime );
 
 			ImGui::Begin( "Overlay", nullptr,
 						  ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoInputs | ImGuiWindowFlags_AlwaysAutoResize |
@@ -748,21 +769,14 @@ int main( int, char** )
 
 		// Limit frame rate to 60Hz
 		double time2 = glfwGetTime();
-		double targetTime = time1 + 1.0f / 60.0f;
-		int loopCount = 0;
+		double targetTime = time1 + 1.0 / 60.0;
 		while ( time2 < targetTime )
 		{
 			b2Yield();
 			time2 = glfwGetTime();
-			++loopCount;
 		}
 
-		frameTime = (float)( time2 - time1 );
-		// if (frame % 17 == 0)
-		//{
-		//	printf("loop count = %d, frame time = %.1f\n", loopCount, 1000.0f * frameTime);
-		// }
-		++frame;
+		frameTime = float( time2 - time1 );
 	}
 
 	delete s_sample;
@@ -775,7 +789,7 @@ int main( int, char** )
 
 	s_settings.Save();
 
-#if defined( _WIN32 )
+#if defined( _MSC_VER )
 	_CrtDumpMemoryLeaks();
 #endif
 
