@@ -1,30 +1,50 @@
 // SPDX-FileCopyrightText: 2023 Erin Catto
 // SPDX-License-Identifier: MIT
 
+#if defined( _MSC_VER ) && !defined( _CRT_SECURE_NO_WARNINGS )
+#define _CRT_SECURE_NO_WARNINGS
+#endif
+
 #include "sample.h"
 
+#include "TaskScheduler.h"
 #include "draw.h"
+#include "imgui.h"
+#include "random.h"
 #include "settings.h"
 
 #include "box2d/box2d.h"
-#include "box2d/collision.h"
 #include "box2d/math_functions.h"
 
 #include <GLFW/glfw3.h>
+#include <ctype.h>
 #include <stdio.h>
-#include <string.h>
+
+class SampleTask : public enki::ITaskSet
+{
+public:
+	SampleTask() = default;
+
+	void ExecuteRange( enki::TaskSetPartition range, uint32_t threadIndex ) override
+	{
+		m_task( range.start, range.end, threadIndex, m_taskContext );
+	}
+
+	b2TaskCallback* m_task = nullptr;
+	void* m_taskContext = nullptr;
+};
 
 static void* EnqueueTask( b2TaskCallback* task, int32_t itemCount, int32_t minRange, void* taskContext, void* userContext )
 {
 	Sample* sample = static_cast<Sample*>( userContext );
-	if ( sample->m_taskCount < maxTasks )
+	if ( sample->m_taskCount < Sample::m_maxTasks )
 	{
 		SampleTask& sampleTask = sample->m_tasks[sample->m_taskCount];
 		sampleTask.m_SetSize = itemCount;
 		sampleTask.m_MinRange = minRange;
 		sampleTask.m_task = task;
 		sampleTask.m_taskContext = taskContext;
-		sample->m_scheduler.AddTaskSetToPipe( &sampleTask );
+		sample->m_scheduler->AddTaskSetToPipe( &sampleTask );
 		++sample->m_taskCount;
 		return &sampleTask;
 	}
@@ -43,7 +63,7 @@ static void FinishTask( void* taskPtr, void* userContext )
 	{
 		SampleTask* sampleTask = static_cast<SampleTask*>( taskPtr );
 		Sample* sample = static_cast<Sample*>( userContext );
-		sample->m_scheduler.WaitforTask( sampleTask );
+		sample->m_scheduler->WaitforTask( sampleTask );
 	}
 }
 
@@ -77,21 +97,18 @@ static void TestMathCpp()
 
 Sample::Sample( Settings& settings )
 {
-	m_scheduler.Initialize( settings.workerCount );
+	m_scheduler = new enki::TaskScheduler;
+	m_scheduler->Initialize( settings.workerCount );
+
+	m_tasks = new SampleTask[m_maxTasks];
 	m_taskCount = 0;
 
 	m_threadCount = 1 + settings.workerCount;
 
-	b2WorldDef worldDef = b2DefaultWorldDef();
-	worldDef.workerCount = settings.workerCount;
-	worldDef.enqueueTask = EnqueueTask;
-	worldDef.finishTask = FinishTask;
-	worldDef.userTaskContext = this;
-	worldDef.enableSleep = settings.enableSleep;
+	m_worldId = b2_nullWorldId;
 
-	m_worldId = b2CreateWorld( &worldDef );
 	m_textLine = 30;
-	m_textIncrement = 18;
+	m_textIncrement = 22;
 	m_mouseJointId = b2_nullJointId;
 
 	m_stepCount = 0;
@@ -101,6 +118,11 @@ Sample::Sample( Settings& settings )
 	m_maxProfile = {};
 	m_totalProfile = {};
 
+	g_seed = RAND_SEED;
+
+	m_settings = &settings;
+
+	CreateWorld();
 	TestMathCpp();
 }
 
@@ -108,6 +130,26 @@ Sample::~Sample()
 {
 	// By deleting the world, we delete the bomb, mouse joint, etc.
 	b2DestroyWorld( m_worldId );
+
+	delete m_scheduler;
+	delete[] m_tasks;
+}
+
+void Sample::CreateWorld()
+{
+	if ( B2_IS_NON_NULL( m_worldId ) )
+	{
+		b2DestroyWorld( m_worldId );
+		m_worldId = b2_nullWorldId;
+	}
+
+	b2WorldDef worldDef = b2DefaultWorldDef();
+	worldDef.workerCount = m_settings->workerCount;
+	worldDef.enqueueTask = EnqueueTask;
+	worldDef.finishTask = FinishTask;
+	worldDef.userTaskContext = this;
+	worldDef.enableSleep = m_settings->enableSleep;
+	m_worldId = b2CreateWorld( &worldDef );
 }
 
 void Sample::DrawTitle( const char* string )
@@ -217,6 +259,23 @@ void Sample::MouseMove( b2Vec2 p )
 	}
 }
 
+void Sample::DrawTextLine( const char* text, ... )
+{
+	va_list arg;
+	va_start( arg, text );
+	ImGui::Begin( "Overlay", nullptr,
+				  ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoInputs | ImGuiWindowFlags_AlwaysAutoResize |
+					  ImGuiWindowFlags_NoScrollbar );
+	ImGui::PushFont( g_draw.m_regularFont );
+	ImGui::SetCursorPos( ImVec2( 5.0f, float( m_textLine ) ) );
+	ImGui::TextColoredV( ImColor( 230, 153, 153, 255 ), text, arg );
+	ImGui::PopFont();
+	ImGui::End();
+	va_end( arg );
+
+	m_textLine += m_textIncrement;
+}
+
 void Sample::ResetProfile()
 {
 	m_totalProfile = {};
@@ -239,8 +298,11 @@ void Sample::Step( Settings& settings )
 			timeStep = 0.0f;
 		}
 
-		g_draw.DrawString( 5, m_textLine, "****PAUSED****" );
-		m_textLine += m_textIncrement;
+		if ( g_draw.m_showUI )
+		{
+			g_draw.DrawString( 5, m_textLine, "****PAUSED****" );
+			m_textLine += m_textIncrement;
+		}
 	}
 
 	g_draw.m_debugDraw.drawingBounds = g_camera.GetViewBounds();
@@ -255,13 +317,16 @@ void Sample::Step( Settings& settings )
 	g_draw.m_debugDraw.drawShapes = settings.drawShapes;
 	g_draw.m_debugDraw.drawJoints = settings.drawJoints;
 	g_draw.m_debugDraw.drawJointExtras = settings.drawJointExtras;
-	g_draw.m_debugDraw.drawAABBs = settings.drawAABBs;
+	g_draw.m_debugDraw.drawBounds = settings.drawBounds;
 	g_draw.m_debugDraw.drawMass = settings.drawMass;
+	g_draw.m_debugDraw.drawBodyNames = settings.drawBodyNames;
 	g_draw.m_debugDraw.drawContacts = settings.drawContactPoints;
 	g_draw.m_debugDraw.drawGraphColors = settings.drawGraphColors;
 	g_draw.m_debugDraw.drawContactNormals = settings.drawContactNormals;
 	g_draw.m_debugDraw.drawContactImpulses = settings.drawContactImpulses;
+	g_draw.m_debugDraw.drawContactFeatures = settings.drawContactFeatures;
 	g_draw.m_debugDraw.drawFrictionImpulses = settings.drawFrictionImpulses;
+	g_draw.m_debugDraw.drawIslands = settings.drawIslands;
 
 	b2World_EnableSleeping( m_worldId, settings.enableSleep );
 	b2World_EnableWarmStarting( m_worldId, settings.enableWarmStarting );
@@ -284,35 +349,25 @@ void Sample::Step( Settings& settings )
 	{
 		b2Counters s = b2World_GetCounters( m_worldId );
 
-		g_draw.DrawString( 5, m_textLine, "bodies/shapes/contacts/joints = %d/%d/%d/%d", s.bodyCount, s.shapeCount,
-						   s.contactCount, s.jointCount );
-		m_textLine += m_textIncrement;
-
-		g_draw.DrawString( 5, m_textLine, "islands/tasks = %d/%d", s.islandCount, s.taskCount );
-		m_textLine += m_textIncrement;
-
-		g_draw.DrawString( 5, m_textLine, "tree height static/movable = %d/%d", s.staticTreeHeight, s.treeHeight );
-		m_textLine += m_textIncrement;
+		DrawTextLine( "bodies/shapes/contacts/joints = %d/%d/%d/%d", s.bodyCount, s.shapeCount, s.contactCount, s.jointCount );
+		DrawTextLine( "islands/tasks = %d/%d", s.islandCount, s.taskCount );
+		DrawTextLine( "tree height static/movable = %d/%d", s.staticTreeHeight, s.treeHeight );
 
 		int totalCount = 0;
 		char buffer[256] = { 0 };
-		static_assert( std::size( s.colorCounts ) == 12 );
+		int colorCount = sizeof( s.colorCounts ) / sizeof( s.colorCounts[0] );
 
+		// todo fix this
 		int offset = snprintf( buffer, 256, "colors: " );
-		for ( int i = 0; i < 12; ++i )
+		for ( int i = 0; i < colorCount; ++i )
 		{
 			offset += snprintf( buffer + offset, 256 - offset, "%d/", s.colorCounts[i] );
 			totalCount += s.colorCounts[i];
 		}
 		snprintf( buffer + offset, 256 - offset, "[%d]", totalCount );
-		g_draw.DrawString( 5, m_textLine, buffer );
-		m_textLine += m_textIncrement;
-
-		g_draw.DrawString( 5, m_textLine, "stack allocator size = %d K", s.stackUsed / 1024 );
-		m_textLine += m_textIncrement;
-
-		g_draw.DrawString( 5, m_textLine, "total allocation = %d K", s.byteCount / 1024 );
-		m_textLine += m_textIncrement;
+		DrawTextLine( buffer );
+		DrawTextLine( "stack allocator size = %d K", s.stackUsed / 1024 );
+		DrawTextLine( "total allocation = %d K", s.byteCount / 1024 );
 	}
 
 	// Track maximum profile times
@@ -322,55 +377,54 @@ void Sample::Step( Settings& settings )
 		m_maxProfile.pairs = b2MaxFloat( m_maxProfile.pairs, p.pairs );
 		m_maxProfile.collide = b2MaxFloat( m_maxProfile.collide, p.collide );
 		m_maxProfile.solve = b2MaxFloat( m_maxProfile.solve, p.solve );
-		m_maxProfile.buildIslands = b2MaxFloat( m_maxProfile.buildIslands, p.buildIslands );
+		m_maxProfile.mergeIslands = b2MaxFloat( m_maxProfile.mergeIslands, p.mergeIslands );
+		m_maxProfile.prepareStages = b2MaxFloat( m_maxProfile.prepareStages, p.prepareStages );
 		m_maxProfile.solveConstraints = b2MaxFloat( m_maxProfile.solveConstraints, p.solveConstraints );
-		m_maxProfile.prepareTasks = b2MaxFloat( m_maxProfile.prepareTasks, p.prepareTasks );
-		m_maxProfile.solverTasks = b2MaxFloat( m_maxProfile.solverTasks, p.solverTasks );
 		m_maxProfile.prepareConstraints = b2MaxFloat( m_maxProfile.prepareConstraints, p.prepareConstraints );
 		m_maxProfile.integrateVelocities = b2MaxFloat( m_maxProfile.integrateVelocities, p.integrateVelocities );
 		m_maxProfile.warmStart = b2MaxFloat( m_maxProfile.warmStart, p.warmStart );
-		m_maxProfile.solveVelocities = b2MaxFloat( m_maxProfile.solveVelocities, p.solveVelocities );
+		m_maxProfile.solveImpulses = b2MaxFloat( m_maxProfile.solveImpulses, p.solveImpulses );
 		m_maxProfile.integratePositions = b2MaxFloat( m_maxProfile.integratePositions, p.integratePositions );
-		m_maxProfile.relaxVelocities = b2MaxFloat( m_maxProfile.relaxVelocities, p.relaxVelocities );
+		m_maxProfile.relaxImpulses = b2MaxFloat( m_maxProfile.relaxImpulses, p.relaxImpulses );
 		m_maxProfile.applyRestitution = b2MaxFloat( m_maxProfile.applyRestitution, p.applyRestitution );
 		m_maxProfile.storeImpulses = b2MaxFloat( m_maxProfile.storeImpulses, p.storeImpulses );
-		m_maxProfile.finalizeBodies = b2MaxFloat( m_maxProfile.finalizeBodies, p.finalizeBodies );
-		m_maxProfile.sleepIslands = b2MaxFloat( m_maxProfile.sleepIslands, p.sleepIslands );
+		m_maxProfile.transforms = b2MaxFloat( m_maxProfile.transforms, p.transforms );
 		m_maxProfile.splitIslands = b2MaxFloat( m_maxProfile.splitIslands, p.splitIslands );
 		m_maxProfile.hitEvents = b2MaxFloat( m_maxProfile.hitEvents, p.hitEvents );
-		m_maxProfile.broadphase = b2MaxFloat( m_maxProfile.broadphase, p.broadphase );
-		m_maxProfile.continuous = b2MaxFloat( m_maxProfile.continuous, p.continuous );
+		m_maxProfile.refit = b2MaxFloat( m_maxProfile.refit, p.refit );
+		m_maxProfile.bullets = b2MaxFloat( m_maxProfile.bullets, p.bullets );
+		m_maxProfile.sleepIslands = b2MaxFloat( m_maxProfile.sleepIslands, p.sleepIslands );
+		m_maxProfile.sensors = b2MaxFloat( m_maxProfile.sensors, p.sensors );
 
 		m_totalProfile.step += p.step;
 		m_totalProfile.pairs += p.pairs;
 		m_totalProfile.collide += p.collide;
 		m_totalProfile.solve += p.solve;
-		m_totalProfile.buildIslands += p.buildIslands;
+		m_totalProfile.mergeIslands += p.mergeIslands;
+		m_totalProfile.prepareStages += p.prepareStages;
 		m_totalProfile.solveConstraints += p.solveConstraints;
-		m_totalProfile.prepareTasks += p.prepareTasks;
-		m_totalProfile.solverTasks += p.solverTasks;
 		m_totalProfile.prepareConstraints += p.prepareConstraints;
 		m_totalProfile.integrateVelocities += p.integrateVelocities;
 		m_totalProfile.warmStart += p.warmStart;
-		m_totalProfile.solveVelocities += p.solveVelocities;
+		m_totalProfile.solveImpulses += p.solveImpulses;
 		m_totalProfile.integratePositions += p.integratePositions;
-		m_totalProfile.relaxVelocities += p.relaxVelocities;
+		m_totalProfile.relaxImpulses += p.relaxImpulses;
 		m_totalProfile.applyRestitution += p.applyRestitution;
 		m_totalProfile.storeImpulses += p.storeImpulses;
-		m_totalProfile.finalizeBodies += p.finalizeBodies;
-		m_totalProfile.sleepIslands += p.sleepIslands;
+		m_totalProfile.transforms += p.transforms;
 		m_totalProfile.splitIslands += p.splitIslands;
 		m_totalProfile.hitEvents += p.hitEvents;
-		m_totalProfile.broadphase += p.broadphase;
-		m_totalProfile.continuous += p.continuous;
+		m_totalProfile.refit += p.refit;
+		m_totalProfile.bullets += p.bullets;
+		m_totalProfile.sleepIslands += p.sleepIslands;
+		m_totalProfile.sensors += p.sensors;
 	}
 
 	if ( settings.drawProfile )
 	{
 		b2Profile p = b2World_GetProfile( m_worldId );
 
-		b2Profile aveProfile;
-		memset( &aveProfile, 0, sizeof( b2Profile ) );
+		b2Profile aveProfile = {};
 		if ( m_stepCount > 0 )
 		{
 			float scale = 1.0f / m_stepCount;
@@ -378,92 +432,63 @@ void Sample::Step( Settings& settings )
 			aveProfile.pairs = scale * m_totalProfile.pairs;
 			aveProfile.collide = scale * m_totalProfile.collide;
 			aveProfile.solve = scale * m_totalProfile.solve;
-			aveProfile.buildIslands = scale * m_totalProfile.buildIslands;
+			aveProfile.mergeIslands = scale * m_totalProfile.mergeIslands;
+			aveProfile.prepareStages = scale * m_totalProfile.prepareStages;
 			aveProfile.solveConstraints = scale * m_totalProfile.solveConstraints;
-			aveProfile.prepareTasks = scale * m_totalProfile.prepareTasks;
-			aveProfile.solverTasks = scale * m_totalProfile.solverTasks;
 			aveProfile.prepareConstraints = scale * m_totalProfile.prepareConstraints;
 			aveProfile.integrateVelocities = scale * m_totalProfile.integrateVelocities;
 			aveProfile.warmStart = scale * m_totalProfile.warmStart;
-			aveProfile.solveVelocities = scale * m_totalProfile.solveVelocities;
+			aveProfile.solveImpulses = scale * m_totalProfile.solveImpulses;
 			aveProfile.integratePositions = scale * m_totalProfile.integratePositions;
-			aveProfile.relaxVelocities = scale * m_totalProfile.relaxVelocities;
+			aveProfile.relaxImpulses = scale * m_totalProfile.relaxImpulses;
 			aveProfile.applyRestitution = scale * m_totalProfile.applyRestitution;
 			aveProfile.storeImpulses = scale * m_totalProfile.storeImpulses;
-			aveProfile.finalizeBodies = scale * m_totalProfile.finalizeBodies;
-			aveProfile.sleepIslands = scale * m_totalProfile.sleepIslands;
+			aveProfile.transforms = scale * m_totalProfile.transforms;
 			aveProfile.splitIslands = scale * m_totalProfile.splitIslands;
 			aveProfile.hitEvents = scale * m_totalProfile.hitEvents;
-			aveProfile.broadphase = scale * m_totalProfile.broadphase;
-			aveProfile.continuous = scale * m_totalProfile.continuous;
+			aveProfile.refit = scale * m_totalProfile.refit;
+			aveProfile.bullets = scale * m_totalProfile.bullets;
+			aveProfile.sleepIslands = scale * m_totalProfile.sleepIslands;
+			aveProfile.sensors = scale * m_totalProfile.sensors;
 		}
 
-		g_draw.DrawString( 5, m_textLine, "step [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.step, aveProfile.step,
-						   m_maxProfile.step );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "pairs [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.pairs, aveProfile.pairs,
-						   m_maxProfile.pairs );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "collide [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.collide, aveProfile.collide,
-						   m_maxProfile.collide );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "solve [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solve, aveProfile.solve,
-						   m_maxProfile.solve );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "builds island [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.buildIslands,
-						   aveProfile.buildIslands, m_maxProfile.buildIslands );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "solve constraints [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solveConstraints,
-						   aveProfile.solveConstraints, m_maxProfile.solveConstraints );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "prepare tasks [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.prepareTasks,
-						   aveProfile.prepareTasks, m_maxProfile.prepareTasks );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "solver tasks [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solverTasks,
-						   aveProfile.solverTasks, m_maxProfile.solverTasks );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "prepare constraints [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.prepareConstraints,
-						   aveProfile.prepareConstraints, m_maxProfile.prepareConstraints );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "integrate velocities [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.integrateVelocities,
-						   aveProfile.integrateVelocities, m_maxProfile.integrateVelocities );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "warm start [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.warmStart, aveProfile.warmStart,
-						   m_maxProfile.warmStart );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "solve velocities [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solveVelocities,
-						   aveProfile.solveVelocities, m_maxProfile.solveVelocities );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "integrate positions [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.integratePositions,
-						   aveProfile.integratePositions, m_maxProfile.integratePositions );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "relax velocities [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.relaxVelocities,
-						   aveProfile.relaxVelocities, m_maxProfile.relaxVelocities );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "apply restitution [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.applyRestitution,
-						   aveProfile.applyRestitution, m_maxProfile.applyRestitution );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "store impulses [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.storeImpulses,
-						   aveProfile.storeImpulses, m_maxProfile.storeImpulses );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "finalize bodies [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.finalizeBodies,
-						   aveProfile.finalizeBodies, m_maxProfile.finalizeBodies );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "sleep islands [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.sleepIslands,
-						   aveProfile.sleepIslands, m_maxProfile.sleepIslands );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "split islands [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.splitIslands,
-						   aveProfile.splitIslands, m_maxProfile.splitIslands );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "hit events [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.hitEvents, aveProfile.hitEvents,
-						   m_maxProfile.hitEvents );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "broad-phase [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.broadphase, aveProfile.broadphase,
-						   m_maxProfile.broadphase );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "continuous collision [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.continuous,
-						   aveProfile.continuous, m_maxProfile.continuous );
-		m_textLine += m_textIncrement;
+		DrawTextLine( "step [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.step, aveProfile.step, m_maxProfile.step );
+		DrawTextLine( "pairs [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.pairs, aveProfile.pairs, m_maxProfile.pairs );
+		DrawTextLine( "collide [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.collide, aveProfile.collide, m_maxProfile.collide );
+		DrawTextLine( "solve [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solve, aveProfile.solve, m_maxProfile.solve );
+		DrawTextLine( "> merge islands [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.mergeIslands, aveProfile.mergeIslands,
+					  m_maxProfile.mergeIslands );
+		DrawTextLine( "> prepare tasks [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.prepareStages, aveProfile.prepareStages,
+					  m_maxProfile.prepareStages );
+		DrawTextLine( "> solve constraints [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solveConstraints, aveProfile.solveConstraints,
+					  m_maxProfile.solveConstraints );
+		DrawTextLine( ">> prepare constraints [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.prepareConstraints,
+					  aveProfile.prepareConstraints, m_maxProfile.prepareConstraints );
+		DrawTextLine( ">> integrate velocities [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.integrateVelocities,
+					  aveProfile.integrateVelocities, m_maxProfile.integrateVelocities );
+		DrawTextLine( ">> warm start [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.warmStart, aveProfile.warmStart,
+					  m_maxProfile.warmStart );
+		DrawTextLine( ">> solve impulses [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.solveImpulses, aveProfile.solveImpulses,
+					  m_maxProfile.solveImpulses );
+		DrawTextLine( ">> integrate positions [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.integratePositions,
+					  aveProfile.integratePositions, m_maxProfile.integratePositions );
+		DrawTextLine( ">> relax impulses [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.relaxImpulses, aveProfile.relaxImpulses,
+					  m_maxProfile.relaxImpulses );
+		DrawTextLine( ">> apply restitution [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.applyRestitution, aveProfile.applyRestitution,
+					  m_maxProfile.applyRestitution );
+		DrawTextLine( ">> store impulses [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.storeImpulses, aveProfile.storeImpulses,
+					  m_maxProfile.storeImpulses );
+		DrawTextLine( ">> split islands [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.splitIslands, aveProfile.splitIslands,
+					  m_maxProfile.splitIslands );
+		DrawTextLine( "> update transforms [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.transforms, aveProfile.transforms,
+					  m_maxProfile.transforms );
+		DrawTextLine( "> hit events [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.hitEvents, aveProfile.hitEvents,
+					  m_maxProfile.hitEvents );
+		DrawTextLine( "> refit BVH [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.refit, aveProfile.refit, m_maxProfile.refit );
+		DrawTextLine( "> sleep islands [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.sleepIslands, aveProfile.sleepIslands,
+					  m_maxProfile.sleepIslands );
+		DrawTextLine( "> bullets [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.bullets, aveProfile.bullets, m_maxProfile.bullets );
+		DrawTextLine( "sensors [ave] (max) = %5.2f [%6.2f] (%6.2f)", p.sensors, aveProfile.sensors, m_maxProfile.sensors );
 	}
 }
 
@@ -472,6 +497,143 @@ void Sample::ShiftOrigin( b2Vec2 newOrigin )
 	// m_world->ShiftOrigin(newOrigin);
 }
 
+// Parse an SVG path element with only straight lines. Example:
+// "M 47.625004,185.20833 H 161.39585 l 29.10417,-2.64583 26.45834,-7.9375 26.45833,-13.22917 23.81251,-21.16666 h "
+// "13.22916 v 44.97916 H 592.66669 V 0 h 21.16671 v 206.375 l -566.208398,-1e-5 z"
+int Sample::ParsePath( const char* svgPath, b2Vec2 offset, b2Vec2* points, int capacity, float scale, bool reverseOrder )
+{
+	int pointCount = 0;
+	b2Vec2 currentPoint = {};
+	const char* ptr = svgPath;
+	char command = *ptr;
+
+	while ( *ptr != '\0' )
+	{
+		if ( isdigit( *ptr ) == 0 && *ptr != '-' )
+		{
+			// note: command can be implicitly repeated
+			command = *ptr;
+
+			if ( command == 'M' || command == 'L' || command == 'H' || command == 'V' || command == 'm' || command == 'l' ||
+				 command == 'h' || command == 'v' )
+			{
+				ptr += 2; // Skip the command character and space
+			}
+
+			if ( command == 'z' )
+			{
+				break;
+			}
+		}
+
+		assert( isdigit( *ptr ) != 0 || *ptr == '-' );
+
+		float x = 0.0f, y = 0.0f;
+		switch ( command )
+		{
+			case 'M':
+			case 'L':
+				if ( sscanf( ptr, "%f,%f", &x, &y ) == 2 )
+				{
+					currentPoint.x = x;
+					currentPoint.y = y;
+				}
+				else
+				{
+					assert( false );
+				}
+				break;
+			case 'H':
+				if ( sscanf( ptr, "%f", &x ) == 1 )
+				{
+					currentPoint.x = x;
+				}
+				else
+				{
+					assert( false );
+				}
+				break;
+			case 'V':
+				if ( sscanf( ptr, "%f", &y ) == 1 )
+				{
+					currentPoint.y = y;
+				}
+				else
+				{
+					assert( false );
+				}
+				break;
+			case 'm':
+			case 'l':
+				if ( sscanf( ptr, "%f,%f", &x, &y ) == 2 )
+				{
+					currentPoint.x += x;
+					currentPoint.y += y;
+				}
+				else
+				{
+					assert( false );
+				}
+				break;
+			case 'h':
+				if ( sscanf( ptr, "%f", &x ) == 1 )
+				{
+					currentPoint.x += x;
+				}
+				else
+				{
+					assert( false );
+				}
+				break;
+			case 'v':
+				if ( sscanf( ptr, "%f", &y ) == 1 )
+				{
+					currentPoint.y += y;
+				}
+				else
+				{
+					assert( false );
+				}
+				break;
+
+			default:
+				assert( false );
+				break;
+		}
+
+		points[pointCount] = { scale * ( currentPoint.x + offset.x ), -scale * ( currentPoint.y + offset.y ) };
+		pointCount += 1;
+		if ( pointCount == capacity )
+		{
+			break;
+		}
+
+		// Move to the next space or end of string
+		while ( *ptr != '\0' && isspace( *ptr ) == 0 )
+		{
+			ptr++;
+		}
+
+		// Skip contiguous spaces
+		while ( isspace( *ptr ) )
+		{
+			ptr++;
+		}
+
+		ptr += 0;
+	}
+
+	if ( pointCount == 0 )
+	{
+		return 0;
+	}
+
+	if ( reverseOrder )
+	{
+	}
+	return pointCount;
+}
+
 SampleEntry g_sampleEntries[MAX_SAMPLES] = {};
 int g_sampleCount = 0;
 
@@ -487,21 +649,3 @@ int RegisterSample( const char* category, const char* name, SampleCreateFcn* fcn
 
 	return -1;
 }
-
-b2Polygon RandomPolygon( float extent )
-{
-	b2Vec2 points[b2_maxPolygonVertices];
-	int count = 3 + rand() % 6;
-	for ( int i = 0; i < count; ++i )
-	{
-		points[i] = RandomVec2( -extent, extent );
-	}
-
-	b2Hull hull = b2ComputeHull( points, count );
-	if ( hull.count > 0 )
-	{
-		return b2MakePolygon( &hull, 0.0f );
-	}
-
-	return b2MakeSquare( extent );
-}
