@@ -69,10 +69,9 @@ public:
 			bodyDef.position = { -4.0f, 5.0f };
 			m_platformId = b2CreateBody( m_worldId, &bodyDef );
 
-			b2Polygon box = b2MakeOffsetBox( 0.5f, 4.0f, { 4.0f, 0.0f }, 0.5f * b2_pi );
+			b2Polygon box = b2MakeOffsetBox( 0.5f, 4.0f, { 4.0f, 0.0f }, b2MakeRot( 0.5f * B2_PI ) );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.6f;
 			shapeDef.density = 2.0f;
 			b2CreatePolygonShape( m_platformId, &shapeDef, &box );
 
@@ -124,7 +123,6 @@ public:
 			b2Polygon box = b2MakeBox( 0.75f, 0.75f );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.6f;
 			shapeDef.density = 2.0f;
 
 			b2CreatePolygonShape( bodyId, &shapeDef, &box );
@@ -141,7 +139,6 @@ public:
 			b2Polygon box = b2MakeBox( 0.75f, 0.75f );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.6f;
 			shapeDef.density = 2.0f;
 
 			b2CreatePolygonShape( m_secondPayloadId, &shapeDef, &box );
@@ -158,7 +155,6 @@ public:
 			b2Capsule capsule = { { 0.0f, 0.0f }, { 1.0f, 0.0f }, 0.25f };
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.6f;
 			shapeDef.density = 2.0f;
 
 			b2CreateCapsuleShape( m_touchingBodyId, &shapeDef, &capsule );
@@ -176,14 +172,13 @@ public:
 			b2Circle circle = { { 0.0f, 0.5f }, 0.25f };
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.6f;
 			shapeDef.density = 2.0f;
 
 			b2CreateCircleShape( m_floatingBodyId, &shapeDef, &circle );
 		}
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 140.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -287,188 +282,15 @@ public:
 
 static int sampleBodyType = RegisterSample( "Bodies", "Body Type", BodyType::Create );
 
-/// This is a test of typical character collision scenarios. This does not
-/// show how you should implement a character in your application.
-/// Instead this is used to test smooth collision on edge chains.
-class Character : public Sample
+float FrictionCallback( float, int, float, int )
 {
-public:
-	explicit Character( Settings& settings )
-		: Sample( settings )
-	{
-		if ( settings.restart == false )
-		{
-			g_camera.m_center = { -2.0f, 7.0f };
-			g_camera.m_zoom = 25.0f * 0.4f;
-		}
+	return 0.1f;
+}
 
-		// Ground body
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Segment segment = { { -20.0f, 0.0f }, { 20.0f, 0.0f } };
-			b2CreateSegmentShape( groundId, &shapeDef, &segment );
-		}
-
-		// Collinear edges with no adjacency information.
-		// This shows the problematic case where a box shape can hit
-		// an internal vertex.
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Segment segment1 = { { -8.0f, 1.0f }, { -6.0f, 1.0f } };
-			b2CreateSegmentShape( groundId, &shapeDef, &segment1 );
-
-			b2Segment segment2 = { { -6.0f, 1.0f }, { -4.0f, 1.0f } };
-			b2CreateSegmentShape( groundId, &shapeDef, &segment2 );
-
-			b2Segment segment3 = { { -4.0f, 1.0f }, { -2.0f, 1.0f } };
-			b2CreateSegmentShape( groundId, &shapeDef, &segment3 );
-		}
-
-		// Chain shape
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			bodyDef.rotation = b2MakeRot( 0.25f * b2_pi );
-			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2Vec2 points[4] = { { 8.0f, 7.0f }, { 7.0f, 8.0f }, { 6.0f, 8.0f }, { 5.0f, 7.0f } };
-			b2ChainDef chainDef = b2DefaultChainDef();
-			chainDef.points = points;
-			chainDef.count = 4;
-			chainDef.isLoop = true;
-
-			b2CreateChain( groundId, &chainDef );
-		}
-
-		// Square tiles. This shows that adjacency shapes may have non-smooth collision. Box2D has no solution
-		// to this problem.
-		// TODO_ERIN try this: https://briansemrau.github.io/dealing-with-ghost-collisions/
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 1.0f, 1.0f, { 4.0f, 3.0f }, 0.0f );
-			b2CreatePolygonShape( groundId, &shapeDef, &box );
-
-			box = b2MakeOffsetBox( 1.0f, 1.0f, { 6.0f, 3.0f }, 0.0f );
-			b2CreatePolygonShape( groundId, &shapeDef, &box );
-
-			box = b2MakeOffsetBox( 1.0f, 1.0f, { 8.0f, 3.0f }, 0.0f );
-			b2CreatePolygonShape( groundId, &shapeDef, &box );
-		}
-
-		// Square made from a chain loop. Collision should be smooth.
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2Vec2 points[4] = { { -1.0f, 3.0 }, { 1.0f, 3.0f }, { 1.0f, 5.0f }, { -1.0f, 5.0 } };
-			b2ChainDef chainDef = b2DefaultChainDef();
-			chainDef.points = points;
-			chainDef.count = 4;
-			chainDef.isLoop = true;
-			b2CreateChain( groundId, &chainDef );
-		}
-
-		// Chain loop. Collision should be smooth.
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			bodyDef.position = { -10.0f, 4.0f };
-			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2Vec2 points[10] = { { 0.0f, 0.0f }, { 6.0f, 0.0f },  { 6.0f, 2.0f },	{ 4.0f, 1.0f },	 { 2.0f, 2.0f },
-								  { 0.0f, 2.0f }, { -2.0f, 2.0f }, { -4.0f, 3.0f }, { -6.0f, 2.0f }, { -6.0f, 0.0f } };
-			b2ChainDef chainDef = b2DefaultChainDef();
-			chainDef.points = points;
-			chainDef.count = 10;
-			chainDef.isLoop = true;
-			b2CreateChain( groundId, &chainDef );
-		}
-
-		// Circle character
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			bodyDef.position = { -7.0f, 6.0f };
-			bodyDef.type = b2_dynamicBody;
-			bodyDef.fixedRotation = true;
-			bodyDef.enableSleep = false;
-
-			m_circleCharacterId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2Circle circle = { { 0.0f, 0.0f }, 0.25f };
-
-			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.density = 20.0f;
-			shapeDef.friction = 0.2f;
-			b2CreateCircleShape( m_circleCharacterId, &shapeDef, &circle );
-		}
-
-		// Capsule character
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			bodyDef.position = { 3.0f, 5.0f };
-			bodyDef.type = b2_dynamicBody;
-			bodyDef.fixedRotation = true;
-			bodyDef.enableSleep = false;
-
-			m_capsuleCharacterId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2Capsule capsule = { { 0.0f, 0.25f }, { 0.0f, 0.75f }, 0.25f };
-
-			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.density = 20.0f;
-			shapeDef.friction = 0.2f;
-			b2CreateCapsuleShape( m_capsuleCharacterId, &shapeDef, &capsule );
-		}
-
-		// Square character
-		{
-			b2BodyDef bodyDef = b2DefaultBodyDef();
-			bodyDef.position = { -3.0f, 8.0f };
-			bodyDef.type = b2_dynamicBody;
-			bodyDef.fixedRotation = true;
-			bodyDef.enableSleep = false;
-
-			m_boxCharacterId = b2CreateBody( m_worldId, &bodyDef );
-
-			b2Polygon box = b2MakeBox( 0.4f, 0.4f );
-
-			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.density = 20.0f;
-			shapeDef.friction = 0.2f;
-			b2CreatePolygonShape( m_boxCharacterId, &shapeDef, &box );
-		}
-	}
-
-	void Step( Settings& settings ) override
-	{
-		Sample::Step( settings );
-
-		g_draw.DrawString( 5, m_textLine, "This tests various character collision shapes." );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "Limitation: square and hexagon can snag on aligned boxes." );
-		m_textLine += m_textIncrement;
-		g_draw.DrawString( 5, m_textLine, "Feature: edge chains have smooth collision inside and out." );
-		m_textLine += m_textIncrement;
-	}
-
-	static Sample* Create( Settings& settings )
-	{
-		return new Character( settings );
-	}
-
-	b2BodyId m_circleCharacterId;
-	b2BodyId m_capsuleCharacterId;
-	b2BodyId m_boxCharacterId;
-};
-
-static int sampleCharacter = RegisterSample( "Bodies", "Character", Character::Create );
+float RestitutionCallback( float, int, float, int )
+{
+	return 1.0f;
+}
 
 class Weeble : public Sample
 {
@@ -482,6 +304,10 @@ public:
 			g_camera.m_zoom = 25.0f * 0.5f;
 		}
 
+		// Test friction and restitution callbacks
+		b2World_SetFrictionCallback( m_worldId, FrictionCallback );
+		b2World_SetRestitutionCallback( m_worldId, RestitutionCallback );
+
 		b2BodyId groundId = b2_nullBodyId;
 		{
 			b2BodyDef bodyDef = b2DefaultBodyDef();
@@ -497,16 +323,15 @@ public:
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			bodyDef.type = b2_dynamicBody;
 			bodyDef.position = { 0.0f, 3.0f };
-			bodyDef.rotation = b2MakeRot( 0.25f * b2_pi );
+			bodyDef.rotation = b2MakeRot( 0.25f * B2_PI );
 			m_weebleId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2Capsule capsule = { { 0.0f, -1.0f }, { 0.0f, 1.0f }, 1.0f };
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.density = 1.0f;
 			b2CreateCapsuleShape( m_weebleId, &shapeDef, &capsule );
 
 			float mass = b2Body_GetMass( m_weebleId );
-			float inertiaTensor = b2Body_GetInertiaTensor( m_weebleId );
+			float inertiaTensor = b2Body_GetRotationalInertia( m_weebleId );
 
 			float offset = 1.5f;
 
@@ -522,7 +347,7 @@ public:
 		m_explosionMagnitude = 8.0f;
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 120.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -530,12 +355,17 @@ public:
 		ImGui::Begin( "Weeble", nullptr, ImGuiWindowFlags_NoMove | ImGuiWindowFlags_NoResize );
 		if ( ImGui::Button( "Teleport" ) )
 		{
-			b2Body_SetTransform( m_weebleId, { 0.0f, 5.0f }, b2MakeRot( 0.95 * b2_pi ) );
+			b2Body_SetTransform( m_weebleId, { 0.0f, 5.0f }, b2MakeRot( 0.95 * B2_PI ) );
 		}
 
 		if ( ImGui::Button( "Explode" ) )
 		{
-			b2World_Explode( m_worldId, m_explosionPosition, m_explosionRadius, m_explosionMagnitude );
+			b2ExplosionDef def = b2DefaultExplosionDef();
+			def.position = m_explosionPosition;
+			def.radius = m_explosionRadius;
+			def.falloff = 0.1f;
+			def.impulsePerLength = m_explosionMagnitude;
+			b2World_Explode( m_worldId, &def );
 		}
 		ImGui::PushItemWidth( 100.0f );
 
@@ -550,6 +380,17 @@ public:
 		Sample::Step( settings );
 
 		g_draw.DrawCircle( m_explosionPosition, m_explosionRadius, b2_colorAzure );
+
+		// This shows how to get the velocity of a point on a body
+		b2Vec2 localPoint = { 0.0f, 2.0f };
+		b2Vec2 worldPoint = b2Body_GetWorldPoint( m_weebleId, localPoint );
+
+		b2Vec2 v1 = b2Body_GetLocalPointVelocity( m_weebleId, localPoint );
+		b2Vec2 v2 = b2Body_GetWorldPointVelocity( m_weebleId, worldPoint );
+
+		b2Vec2 offset = { 0.05f, 0.0f };
+		g_draw.DrawSegment( worldPoint, worldPoint + v1, b2_colorRed );
+		g_draw.DrawSegment( worldPoint + offset, worldPoint + v2 + offset, b2_colorGreen );
 	}
 
 	static Sample* Create( Settings& settings )
@@ -582,8 +423,9 @@ public:
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			groundId = b2CreateBody( m_worldId, &bodyDef );
 
-			b2Segment segment = { { -20.0f, 0.0f }, { 20.0f, 0.0f } };
+			b2Segment segment = { { -40.0f, 0.0f }, { 40.0f, 0.0f } };
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			shapeDef.enableSensorEvents = true;
 			m_groundShapeId = b2CreateSegmentShape( groundId, &shapeDef, &segment );
 		}
 
@@ -602,6 +444,7 @@ public:
 			b2CreateCapsuleShape( bodyId, &shapeDef, &capsule );
 
 			shapeDef.isSensor = true;
+			shapeDef.enableSensorEvents = true;
 			capsule.radius = 1.0f;
 			m_sensorIds[i] = b2CreateCapsuleShape( bodyId, &shapeDef, &capsule );
 			m_sensorTouching[i] = false;
@@ -630,7 +473,7 @@ public:
 			bodyDef.enableSleep = false;
 			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
 
-			b2Polygon box = b2MakeOffsetBox( 1.0f, 1.0f, { 0.0f, 1.0f }, 0.25f * b2_pi );
+			b2Polygon box = b2MakeOffsetBox( 1.0f, 1.0f, { 0.0f, 1.0f }, b2MakeRot( 0.25f * B2_PI ) );
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
 			b2CreatePolygonShape( bodyId, &shapeDef, &box );
 		}
@@ -670,11 +513,47 @@ public:
 			jointDef.localAnchorB = b2Body_GetLocalPoint( jointDef.bodyIdB, pivot );
 			b2CreateRevoluteJoint( m_worldId, &jointDef );
 		}
+
+		// A sleeping body to test waking on contact destroyed
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.type = b2_dynamicBody;
+			bodyDef.position = { -10.0f, 1.0f };
+			bodyDef.isAwake = false;
+			bodyDef.enableSleep = true;
+			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Polygon box = b2MakeSquare( 1.0f );
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2CreatePolygonShape( bodyId, &shapeDef, &box );
+		}
+
+		m_staticBodyId = b2_nullBodyId;
 	}
 
-	void UpdateUI() override
+	void ToggleInvoker()
 	{
-		float height = 100.0f;
+		if ( B2_IS_NULL( m_staticBodyId ) )
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.position = { -10.5f, 3.0f };
+			m_staticBodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Polygon box = b2MakeOffsetBox( 2.0f, 0.1f, { 0.0f, 0.0f }, b2MakeRot( 0.25f * B2_PI ) );
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			shapeDef.invokeContactCreation = true;
+			b2CreatePolygonShape( m_staticBodyId, &shapeDef, &box );
+		}
+		else
+		{
+			b2DestroyBody( m_staticBodyId );
+			m_staticBodyId = b2_nullBodyId;
+		}
+	}
+
+	void UpdateGui() override
+	{
+		float height = 160.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
 		ImGui::SetNextWindowSize( ImVec2( 240.0f, height ) );
 		ImGui::Begin( "Sleep", nullptr, ImGuiWindowFlags_NoMove | ImGuiWindowFlags_NoResize );
@@ -698,6 +577,23 @@ public:
 
 		ImGui::PopItemWidth();
 
+		ImGui::Separator();
+
+		if ( B2_IS_NULL( m_staticBodyId ) )
+		{
+			if ( ImGui::Button( "Create" ) )
+			{
+				ToggleInvoker();
+			}
+		}
+		else
+		{
+			if ( ImGui::Button( "Destroy" ) )
+			{
+				ToggleInvoker();
+			}
+		}
+
 		ImGui::End();
 	}
 
@@ -753,6 +649,7 @@ public:
 	}
 
 	b2BodyId m_pendulumId;
+	b2BodyId m_staticBodyId;
 	b2ShapeId m_groundShapeId;
 	b2ShapeId m_sensorIds[2];
 	bool m_sensorTouching[2];
@@ -787,8 +684,8 @@ public:
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			bodyDef.type = b2_dynamicBody;
 			bodyDef.position = { 0.0f, 3.0f };
-			bodyDef.angularVelocity = 0.2f;
-			bodyDef.rotation = b2MakeRot( 0.25f * b2_pi );
+			bodyDef.angularVelocity = 0.5f;
+			bodyDef.rotation = b2MakeRot( 0.25f * B2_PI );
 
 			m_badBodyId = b2CreateBody( m_worldId, &bodyDef );
 
@@ -805,7 +702,7 @@ public:
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			bodyDef.type = b2_dynamicBody;
 			bodyDef.position = { 2.0f, 3.0f };
-			bodyDef.rotation = b2MakeRot( 0.25f * b2_pi );
+			bodyDef.rotation = b2MakeRot( 0.25f * B2_PI );
 
 			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
 
@@ -839,3 +736,145 @@ public:
 };
 
 static int sampleBadBody = RegisterSample( "Bodies", "Bad", BadBody::Create );
+
+// This shows how to set the initial angular velocity to get a specific movement.
+class Pivot : public Sample
+{
+public:
+	explicit Pivot( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 0.8f, 6.4f };
+			g_camera.m_zoom = 25.0f * 0.4f;
+		}
+
+		b2BodyId groundId = b2_nullBodyId;
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Segment segment = { { -20.0f, 0.0f }, { 20.0f, 0.0f } };
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2CreateSegmentShape( groundId, &shapeDef, &segment );
+		}
+
+		// Create a separate body on the ground
+		{
+			b2Vec2 v = { 5.0f, 0.0f };
+
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.type = b2_dynamicBody;
+			bodyDef.position = { 0.0f, 3.0f };
+			bodyDef.gravityScale = 1.0f;
+			bodyDef.linearVelocity = v;
+
+			m_bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			m_lever = 3.0f;
+			b2Vec2 r = { 0.0f, -m_lever };
+
+			float omega = b2Cross( v, r ) / b2Dot( r, r );
+			b2Body_SetAngularVelocity( m_bodyId, omega );
+
+			b2Polygon box = b2MakeBox( 0.1f, m_lever );
+
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2CreatePolygonShape( m_bodyId, &shapeDef, &box );
+		}
+	}
+
+	void Step( Settings& settings ) override
+	{
+		Sample::Step( settings );
+
+		b2Vec2 v = b2Body_GetLinearVelocity( m_bodyId );
+		float omega = b2Body_GetAngularVelocity( m_bodyId );
+		b2Vec2 r = b2Body_GetWorldVector( m_bodyId, { 0.0f, -m_lever } );
+
+		b2Vec2 vp = v + b2CrossSV( omega, r );
+		g_draw.DrawString( 5, m_textLine, "pivot velocity = (%g, %g)", vp.x, vp.y );
+		m_textLine += m_textIncrement;
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new Pivot( settings );
+	}
+
+	b2BodyId m_bodyId;
+	float m_lever;
+};
+
+static int samplePivot = RegisterSample( "Bodies", "Pivot", Pivot::Create );
+
+// This shows how to drive a kinematic body to reach a target
+class Kinematic : public Sample
+{
+public:
+	explicit Kinematic( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 0.0f, 0.0f };
+			g_camera.m_zoom = 4.0f;
+		}
+
+		m_amplitude = 2.0f;
+
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.type = b2_kinematicBody;
+			bodyDef.position.x = 2.0f * m_amplitude;
+
+			m_bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Polygon box = b2MakeBox( 0.1f, 1.0f );
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2CreatePolygonShape( m_bodyId, &shapeDef, &box );
+		}
+
+		m_time = 0.0f;
+	}
+
+	void Step( Settings& settings ) override
+	{
+		float timeStep = settings.hertz > 0.0f ? 1.0f / settings.hertz : 0.0f;
+		if ( settings.pause && settings.singleStep == false )
+		{
+			timeStep = 0.0f;
+		}
+
+		if ( timeStep > 0.0f )
+		{
+			b2Vec2 point = {
+				.x = 2.0f * m_amplitude * cosf( m_time ),
+				.y = m_amplitude * sinf( 2.0f * m_time ),
+			};
+			b2Rot rotation = b2MakeRot( 2.0f * m_time );
+
+			b2Vec2 axis = b2RotateVector( rotation, { 0.0f, 1.0f } );
+			g_draw.DrawSegment( point - 0.5f * axis, point + 0.5f * axis, b2_colorPlum );
+			g_draw.DrawPoint( point, 10.0f, b2_colorPlum );
+
+			b2Body_SetTargetTransform( m_bodyId, { point, rotation }, timeStep );
+		}
+
+		Sample::Step( settings );
+
+		m_time += timeStep;
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new Kinematic( settings );
+	}
+
+	b2BodyId m_bodyId;
+	float m_amplitude;
+	float m_time;
+};
+
+static int sampleKinematic = RegisterSample( "Bodies", "Kinematic", Kinematic::Create );
