@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: MIT
 
 #include "draw.h"
+#include "random.h"
 #include "sample.h"
 #include "settings.h"
 
@@ -10,6 +11,10 @@
 
 #include <GLFW/glfw3.h>
 #include <imgui.h>
+#include <vector>
+
+// extern "C" int b2_toiCalls;
+// extern "C" int b2_toiHitCount;
 
 class ChainShape : public Sample
 {
@@ -37,6 +42,7 @@ public:
 		m_shapeType = e_circleShape;
 		m_restitution = 0.0f;
 		m_friction = 0.2f;
+
 		CreateScene();
 		Launch();
 	}
@@ -104,11 +110,17 @@ public:
 		// }
 		// printf("};\n");
 
+		b2SurfaceMaterial material = {};
+		material.friction = 0.2f;
+		material.customColor = b2_colorSteelBlue;
+		material.userMaterialId = 42;
+
 		b2ChainDef chainDef = b2DefaultChainDef();
 		chainDef.points = points;
 		chainDef.count = count;
+		chainDef.materials = &material;
+		chainDef.materialCount = 1;
 		chainDef.isLoop = true;
-		chainDef.friction = 0.2f;
 
 		b2BodyDef bodyDef = b2DefaultBodyDef();
 		m_groundId = b2CreateBody( m_worldId, &bodyDef );
@@ -130,8 +142,8 @@ public:
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
 		shapeDef.density = 1.0f;
-		shapeDef.friction = m_friction;
-		shapeDef.restitution = m_restitution;
+		shapeDef.material.friction = m_friction;
+		shapeDef.material.restitution = m_restitution;
 
 		if ( m_shapeType == e_circleShape )
 		{
@@ -149,11 +161,16 @@ public:
 			b2Polygon box = b2MakeBox( h, h );
 			m_shapeId = b2CreatePolygonShape( m_bodyId, &shapeDef, &box );
 		}
+
+		// b2_toiCalls = 0;
+		// b2_toiHitCount = 0;
+
+		m_stepCount = 0;
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
-		float height = 135.0f;
+		float height = 155.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
 		ImGui::SetNextWindowSize( ImVec2( 240.0f, height ) );
 
@@ -192,6 +209,8 @@ public:
 
 		g_draw.DrawSegment( b2Vec2_zero, { 0.5f, 0.0f }, b2_colorRed );
 		g_draw.DrawSegment( b2Vec2_zero, { 0.0f, 0.5f }, b2_colorGreen );
+
+		// DrawTextLine( "toi calls, hits = %d, %d", b2_toiCalls, b2_toiHitCount );
 	}
 
 	static Sample* Create( Settings& settings )
@@ -241,9 +260,9 @@ public:
 			m_table1Id = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon top = b2MakeOffsetBox( 3.0f, 0.5f, { 0.0f, 3.5f }, 0.0f );
-			b2Polygon leftLeg = b2MakeOffsetBox( 0.5f, 1.5f, { -2.5f, 1.5f }, 0.0f );
-			b2Polygon rightLeg = b2MakeOffsetBox( 0.5f, 1.5f, { 2.5f, 1.5f }, 0.0f );
+			b2Polygon top = b2MakeOffsetBox( 3.0f, 0.5f, { 0.0f, 3.5f }, b2Rot_identity );
+			b2Polygon leftLeg = b2MakeOffsetBox( 0.5f, 1.5f, { -2.5f, 1.5f }, b2Rot_identity );
+			b2Polygon rightLeg = b2MakeOffsetBox( 0.5f, 1.5f, { 2.5f, 1.5f }, b2Rot_identity );
 
 			b2CreatePolygonShape( m_table1Id, &shapeDef, &top );
 			b2CreatePolygonShape( m_table1Id, &shapeDef, &leftLeg );
@@ -258,9 +277,9 @@ public:
 			m_table2Id = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon top = b2MakeOffsetBox( 3.0f, 0.5f, { 0.0f, 3.5f }, 0.0f );
-			b2Polygon leftLeg = b2MakeOffsetBox( 0.5f, 2.0f, { -2.5f, 2.0f }, 0.0f );
-			b2Polygon rightLeg = b2MakeOffsetBox( 0.5f, 2.0f, { 2.5f, 2.0f }, 0.0f );
+			b2Polygon top = b2MakeOffsetBox( 3.0f, 0.5f, { 0.0f, 3.5f }, b2Rot_identity );
+			b2Polygon leftLeg = b2MakeOffsetBox( 0.5f, 2.0f, { -2.5f, 2.0f }, b2Rot_identity );
+			b2Polygon rightLeg = b2MakeOffsetBox( 0.5f, 2.0f, { 2.5f, 2.0f }, b2Rot_identity );
 
 			b2CreatePolygonShape( m_table2Id, &shapeDef, &top );
 			b2CreatePolygonShape( m_table2Id, &shapeDef, &leftLeg );
@@ -333,7 +352,7 @@ public:
 			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 4.0f, 0.1f, { 0.0f, 3.0f }, 0.0f );
+			b2Polygon box = b2MakeOffsetBox( 4.0f, 0.1f, { 0.0f, 3.0f }, b2Rot_identity );
 			b2CreatePolygonShape( bodyId, &shapeDef, &box );
 		}
 
@@ -346,7 +365,7 @@ public:
 			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 4.0f, 0.1f, { 0.0f, 3.0f }, 0.0f );
+			b2Polygon box = b2MakeOffsetBox( 4.0f, 0.1f, { 0.0f, 3.0f }, b2Rot_identity );
 			b2CreatePolygonShape( bodyId, &shapeDef, &box );
 		}
 
@@ -379,7 +398,7 @@ public:
 		}
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 100.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -496,7 +515,7 @@ public:
 		}
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 240.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -742,11 +761,6 @@ static int sampleCustomFilter = RegisterSample( "Shapes", "Custom Filter", Custo
 class Restitution : public Sample
 {
 public:
-	enum
-	{
-		e_count = 40
-	};
-
 	enum ShapeType
 	{
 		e_circleShape = 0,
@@ -766,17 +780,12 @@ public:
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
-			float h = 1.0f * e_count;
+			float h = 1.0f * m_count;
 			b2Segment segment = { { -h, 0.0f }, { h, 0.0f } };
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
 			b2CreateSegmentShape( groundId, &shapeDef, &segment );
 		}
 
-		for ( int i = 0; i < e_count; ++i )
-		{
-			m_bodyIds[i] = b2_nullBodyId;
-		}
-
 		m_shapeType = e_circleShape;
 
 		CreateBodies();
@@ -784,7 +793,7 @@ public:
 
 	void CreateBodies()
 	{
-		for ( int i = 0; i < e_count; ++i )
+		for ( int i = 0; i < m_count; ++i )
 		{
 			if ( B2_IS_NON_NULL( m_bodyIds[i] ) )
 			{
@@ -793,23 +802,23 @@ public:
 			}
 		}
 
-		b2Circle circle = { 0 };
+		b2Circle circle = {};
 		circle.radius = 0.5f;
 
 		b2Polygon box = b2MakeBox( 0.5f, 0.5f );
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
 		shapeDef.density = 1.0f;
-		shapeDef.restitution = 0.0f;
+		shapeDef.material.restitution = 0.0f;
 
 		b2BodyDef bodyDef = b2DefaultBodyDef();
 		bodyDef.type = b2_dynamicBody;
 
-		float dr = 1.0f / ( e_count > 1 ? e_count - 1 : 1 );
-		float x = -1.0f * ( e_count - 1 );
+		float dr = 1.0f / ( m_count > 1 ? m_count - 1 : 1 );
+		float x = -1.0f * ( m_count - 1 );
 		float dx = 2.0f;
 
-		for ( int i = 0; i < e_count; ++i )
+		for ( int i = 0; i < m_count; ++i )
 		{
 			bodyDef.position = { x, 40.0f };
 			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
@@ -825,12 +834,12 @@ public:
 				b2CreatePolygonShape( bodyId, &shapeDef, &box );
 			}
 
-			shapeDef.restitution += dr;
+			shapeDef.material.restitution += dr;
 			x += dx;
 		}
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 100.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -860,7 +869,9 @@ public:
 		return new Restitution( settings );
 	}
 
-	b2BodyId m_bodyIds[e_count];
+	static constexpr int m_count = 40;
+
+	b2BodyId m_bodyIds[m_count] = {};
 	ShapeType m_shapeType;
 };
 
@@ -883,24 +894,24 @@ public:
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.2f;
+			shapeDef.material.friction = 0.2f;
 
 			b2Segment segment = { { -40.0f, 0.0f }, { 40.0f, 0.0f } };
 			b2CreateSegmentShape( groundId, &shapeDef, &segment );
 
-			b2Polygon box = b2MakeOffsetBox( 13.0f, 0.25f, { -4.0f, 22.0f }, -0.25f );
+			b2Polygon box = b2MakeOffsetBox( 13.0f, 0.25f, { -4.0f, 22.0f }, b2MakeRot( -0.25f ) );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			box = b2MakeOffsetBox( 0.25f, 1.0f, { 10.5f, 19.0f }, 0.0f );
+			box = b2MakeOffsetBox( 0.25f, 1.0f, { 10.5f, 19.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			box = b2MakeOffsetBox( 13.0f, 0.25f, { 4.0f, 14.0f }, 0.25f );
+			box = b2MakeOffsetBox( 13.0f, 0.25f, { 4.0f, 14.0f }, b2MakeRot( 0.25f ) );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			box = b2MakeOffsetBox( 0.25f, 1.0f, { -10.5f, 11.0f }, 0.0f );
+			box = b2MakeOffsetBox( 0.25f, 1.0f, { -10.5f, 11.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			box = b2MakeOffsetBox( 13.0f, 0.25f, { -4.0f, 6.0f }, -0.25f );
+			box = b2MakeOffsetBox( 13.0f, 0.25f, { -4.0f, 6.0f }, b2MakeRot( -0.25f ) );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 		}
 
@@ -919,7 +930,7 @@ public:
 				bodyDef.position = { -15.0f + 4.0f * i, 28.0f };
 				b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
 
-				shapeDef.friction = friction[i];
+				shapeDef.material.friction = friction[i];
 				b2CreatePolygonShape( bodyId, &shapeDef, &box );
 			}
 		}
@@ -931,7 +942,295 @@ public:
 	}
 };
 
-static int sampleIndex3 = RegisterSample( "Shapes", "Friction", Friction::Create );
+static int sampleFriction = RegisterSample( "Shapes", "Friction", Friction::Create );
+
+class RollingResistance : public Sample
+{
+public:
+	explicit RollingResistance( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 5.0f, 20.0f };
+			g_camera.m_zoom = 27.5f;
+		}
+
+		m_lift = 0.0f;
+		m_resistScale = 0.02f;
+		CreateScene();
+	}
+
+	void CreateScene()
+	{
+		b2Circle circle = { b2Vec2_zero, 0.5f };
+
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+
+		for ( int i = 0; i < 20; ++i )
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Segment segment = { { -40.0f, 2.0f * i }, { 40.0f, 2.0f * i + m_lift } };
+			b2CreateSegmentShape( groundId, &shapeDef, &segment );
+
+			bodyDef.type = b2_dynamicBody;
+			bodyDef.position = { -39.5f, 2.0f * i + 0.75f };
+			bodyDef.angularVelocity = -10.0f;
+			bodyDef.linearVelocity = { 5.0f, 0.0f };
+
+			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+			shapeDef.material.rollingResistance = m_resistScale * i;
+			b2CreateCircleShape( bodyId, &shapeDef, &circle );
+		}
+	}
+
+	void Keyboard( int key ) override
+	{
+		switch ( key )
+		{
+			case GLFW_KEY_1:
+				m_lift = 0.0f;
+				CreateWorld();
+				CreateScene();
+				break;
+
+			case GLFW_KEY_2:
+				m_lift = 5.0f;
+				CreateWorld();
+				CreateScene();
+				break;
+
+			case GLFW_KEY_3:
+				m_lift = -5.0f;
+				CreateWorld();
+				CreateScene();
+				break;
+
+			default:
+				Sample::Keyboard( key );
+				break;
+		}
+	}
+
+	void Step( Settings& settings ) override
+	{
+		Sample::Step( settings );
+
+		for ( int i = 0; i < 20; ++i )
+		{
+			g_draw.DrawString( { -41.5f, 2.0f * i + 1.0f }, "%.2f", m_resistScale * i );
+		}
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new RollingResistance( settings );
+	}
+
+	float m_resistScale;
+	float m_lift;
+};
+
+static int sampleRollingResistance = RegisterSample( "Shapes", "Rolling Resistance", RollingResistance::Create );
+
+class ConveyorBelt : public Sample
+{
+public:
+	explicit ConveyorBelt( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 2.0f, 7.5f };
+			g_camera.m_zoom = 12.0f;
+		}
+
+		// Ground
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2Segment segment = { { -20.0f, 0.0f }, { 20.0f, 0.0f } };
+			b2CreateSegmentShape( groundId, &shapeDef, &segment );
+		}
+
+		// Platform
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.position = { -5.0f, 5.0f };
+			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Polygon box = b2MakeRoundedBox( 10.0f, 0.25f, 0.25f );
+
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			shapeDef.material.friction = 0.8f;
+			shapeDef.material.tangentSpeed = 2.0f;
+
+			b2CreatePolygonShape( bodyId, &shapeDef, &box );
+		}
+
+		// Boxes
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		b2Polygon cube = b2MakeSquare( 0.5f );
+		for ( int i = 0; i < 5; ++i )
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.type = b2_dynamicBody;
+			bodyDef.position = { -10.0f + 2.0f * i, 7.0f };
+			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2CreatePolygonShape( bodyId, &shapeDef, &cube );
+		}
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new ConveyorBelt( settings );
+	}
+};
+
+static int sampleConveyorBelt = RegisterSample( "Shapes", "Conveyor Belt", ConveyorBelt::Create );
+
+class TangentSpeed : public Sample
+{
+public:
+	explicit TangentSpeed( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 60.0f, -15.0f };
+			g_camera.m_zoom = 38.0f;
+		}
+
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			// const char* path = "M 613.8334,185.20833 H 500.06255 L 470.95838,182.5625 444.50004,174.625 418.04171,161.39583 "
+			//				   "394.2292,140.22917 h "
+			//				   "-13.22916 v 44.97916 H 68.791712 V 0 h -21.16671 v 206.375 l 566.208398,-1e-5 z";
+
+			const char* path = "m 613.8334,185.20833 -42.33338,0 h -37.04166 l -34.39581,0 -29.10417,-2.64583 -26.45834,-7.9375 "
+							   "-26.45833,-13.22917 -23.81251,-21.16666 h -13.22916 v 44.97916 H 68.791712 V 0 h -21.16671 v "
+							   "206.375 l 566.208398,-1e-5 z";
+
+			b2Vec2 offset = { -47.375002f, 0.25f };
+
+			float scale = 0.2f;
+			b2Vec2 points[20] = {};
+			int count = ParsePath( path, offset, points, 20, scale, true );
+
+			b2SurfaceMaterial materials[20] = {};
+			for ( int i = 0; i < 20; ++i )
+			{
+				materials[i].friction = 0.6f;
+			}
+
+			materials[0].tangentSpeed = -10.0;
+			materials[0].customColor = b2_colorDarkBlue;
+			materials[1].tangentSpeed = -20.0;
+			materials[1].customColor = b2_colorDarkCyan;
+			materials[2].tangentSpeed = -30.0;
+			materials[2].customColor = b2_colorDarkGoldenRod;
+			materials[3].tangentSpeed = -40.0;
+			materials[3].customColor = b2_colorDarkGray;
+			materials[4].tangentSpeed = -50.0;
+			materials[4].customColor = b2_colorDarkGreen;
+			materials[5].tangentSpeed = -60.0;
+			materials[5].customColor = b2_colorDarkKhaki;
+			materials[6].tangentSpeed = -70.0;
+			materials[6].customColor = b2_colorDarkMagenta;
+
+			b2ChainDef chainDef = b2DefaultChainDef();
+			chainDef.points = points;
+			chainDef.count = count;
+			chainDef.isLoop = true;
+			chainDef.materials = materials;
+			chainDef.materialCount = count;
+
+			b2CreateChain( groundId, &chainDef );
+
+			m_friction = 0.6f;
+			m_rollingResistance = 0.3f;
+		}
+	}
+
+	b2BodyId DropBall()
+	{
+		b2Circle circle = { { 0.0f, 0.0f }, 0.5f };
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		bodyDef.type = b2_dynamicBody;
+		bodyDef.position = { 110.0f, -30.0f };
+		b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		shapeDef.material.friction = m_friction;
+		shapeDef.material.rollingResistance = m_rollingResistance;
+		b2CreateCircleShape( bodyId, &shapeDef, &circle );
+		return bodyId;
+	}
+
+	void Reset()
+	{
+		int count = int( m_bodyIds.size() );
+		for ( int i = 0; i < count; ++i )
+		{
+			b2DestroyBody( m_bodyIds[i] );
+		}
+
+		m_bodyIds.clear();
+	}
+
+	void UpdateGui() override
+	{
+		float height = 80.0f;
+		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
+		ImGui::SetNextWindowSize( ImVec2( 260.0f, height ) );
+
+		ImGui::Begin( "Ball Parameters", nullptr, ImGuiWindowFlags_NoResize );
+		ImGui::PushItemWidth( 140.0f );
+
+		if ( ImGui::SliderFloat( "Friction", &m_friction, 0.0f, 2.0f, "%.2f" ) )
+		{
+			Reset();
+		}
+
+		if ( ImGui::SliderFloat( "Rolling Resistance", &m_rollingResistance, 0.0f, 1.0f, "%.2f" ) )
+		{
+			Reset();
+		}
+
+		ImGui::End();
+	}
+
+	void Step( Settings& settings ) override
+	{
+		if ( m_stepCount % 25 == 0 && m_bodyIds.size() < m_totalCount && settings.pause == false )
+		{
+			b2BodyId id = DropBall();
+			m_bodyIds.push_back( id );
+		}
+
+		Sample::Step( settings );
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new TangentSpeed( settings );
+	}
+
+	static constexpr int m_totalCount = 200;
+	std::vector<b2BodyId> m_bodyIds;
+	float m_friction;
+	float m_rollingResistance;
+};
+
+static int sampleTangentSpeed = RegisterSample( "Shapes", "Tangent Speed", TangentSpeed::Create );
 
 // This sample shows how to modify the geometry on an existing shape. This is only supported on
 // dynamic and kinematic shapes because static shapes don't look for new collisions.
@@ -951,7 +1250,7 @@ public:
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 10.0f, 1.0f, { 0.0f, -1.0f }, 0.0f );
+			b2Polygon box = b2MakeOffsetBox( 10.0f, 1.0f, { 0.0f, -1.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 		}
 
@@ -1011,7 +1310,7 @@ public:
 		b2Body_ApplyMassFromShapes( bodyId );
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 230.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -1193,13 +1492,13 @@ public:
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 20.0f, 1.0f, { 0.0f, -1.0f }, 0.0f );
+			b2Polygon box = b2MakeOffsetBox( 20.0f, 1.0f, { 0.0f, -1.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			box = b2MakeOffsetBox( 1.0f, 5.0f, { 19.0f, 5.0f }, 0.0f );
+			box = b2MakeOffsetBox( 1.0f, 5.0f, { 19.0f, 5.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			box = b2MakeOffsetBox( 1.0f, 5.0f, { -19.0f, 5.0f }, 0.0f );
+			box = b2MakeOffsetBox( 1.0f, 5.0f, { -19.0f, 5.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 		}
 
@@ -1214,6 +1513,7 @@ public:
 		b2BodyDef bodyDef = b2DefaultBodyDef();
 		bodyDef.type = b2_dynamicBody;
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		shapeDef.material.rollingResistance = 0.3f;
 
 		float y = 2.0f;
 		int xcount = 10, ycount = 10;
@@ -1227,7 +1527,7 @@ public:
 				b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
 
 				b2Polygon poly = RandomPolygon( 0.5f );
-				poly.radius = RandomFloat( 0.05f, 0.25f );
+				poly.radius = RandomFloatRange( 0.05f, 0.25f );
 				b2CreatePolygonShape( bodyId, &shapeDef, &poly );
 
 				x += 1.0f;
@@ -1245,6 +1545,71 @@ public:
 
 static int sampleRoundedShapes = RegisterSample( "Shapes", "Rounded", RoundedShapes::Create );
 
+class EllipseShape : public Sample
+{
+public:
+	explicit EllipseShape( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_zoom = 25.0f * 0.55f;
+			g_camera.m_center = { 2.0f, 8.0f };
+		}
+
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2Polygon box = b2MakeOffsetBox( 20.0f, 1.0f, { 0.0f, -1.0f }, b2Rot_identity );
+			b2CreatePolygonShape( groundId, &shapeDef, &box );
+
+			box = b2MakeOffsetBox( 1.0f, 5.0f, { 19.0f, 5.0f }, b2Rot_identity );
+			b2CreatePolygonShape( groundId, &shapeDef, &box );
+
+			box = b2MakeOffsetBox( 1.0f, 5.0f, { -19.0f, 5.0f }, b2Rot_identity );
+			b2CreatePolygonShape( groundId, &shapeDef, &box );
+		}
+
+		b2Vec2 points[6] = {
+			{ 0.0f, -0.25f }, { 0.0f, 0.25f }, { 0.05f, 0.075f }, { -0.05f, 0.075f }, { 0.05f, -0.075f }, { -0.05f, -0.075f },
+		};
+		b2Hull diamondHull = b2ComputeHull( points, 6 );
+		b2Polygon poly = b2MakePolygon( &diamondHull, 0.2f );
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		bodyDef.type = b2_dynamicBody;
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		shapeDef.material.rollingResistance = 0.2f;
+
+		float y = 2.0f;
+		int xCount = 10, yCount = 10;
+
+		for ( int i = 0; i < yCount; ++i )
+		{
+			float x = -5.0f;
+			for ( int j = 0; j < xCount; ++j )
+			{
+				bodyDef.position = { x, y };
+				b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+				b2CreatePolygonShape( bodyId, &shapeDef, &poly );
+
+				x += 1.0f;
+			}
+
+			y += 1.0f;
+		}
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new EllipseShape( settings );
+	}
+};
+
+static int sampleEllipseShape = RegisterSample( "Shapes", "Ellipse", EllipseShape::Create );
+
 class OffsetShapes : public Sample
 {
 public:
@@ -1263,7 +1628,7 @@ public:
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 1.0f, 1.0f, { 10.0f, -2.0f }, 0.5f * b2_pi );
+			b2Polygon box = b2MakeOffsetBox( 1.0f, 1.0f, { 10.0f, -2.0f }, b2MakeRot( 0.5f * B2_PI ) );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 		}
 
@@ -1278,7 +1643,7 @@ public:
 		}
 
 		{
-			b2Polygon box = b2MakeOffsetBox( 0.75f, 0.5f, { 9.0f, 2.0f }, 0.5f * b2_pi );
+			b2Polygon box = b2MakeOffsetBox( 0.75f, 0.5f, { 9.0f, 2.0f }, b2MakeRot( 0.5f * B2_PI ) );
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			bodyDef.position = { 0.0f, 0.0f };
 			bodyDef.type = b2_dynamicBody;
@@ -1302,3 +1667,175 @@ public:
 };
 
 static int sampleOffsetShapes = RegisterSample( "Shapes", "Offset", OffsetShapes::Create );
+
+// This shows how to use explosions and demonstrates the projected perimeter
+class Explosion : public Sample
+{
+public:
+	explicit Explosion( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 0.0f, 0.0f };
+			g_camera.m_zoom = 14.0f;
+		}
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
+
+		bodyDef.type = b2_dynamicBody;
+		bodyDef.gravityScale = 0.0f;
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+
+		m_referenceAngle = 0.0f;
+
+		b2WeldJointDef weldDef = b2DefaultWeldJointDef();
+		weldDef.referenceAngle = m_referenceAngle;
+		weldDef.angularHertz = 0.5f;
+		weldDef.angularDampingRatio = 0.7f;
+		weldDef.linearHertz = 0.5f;
+		weldDef.linearDampingRatio = 0.7f;
+		weldDef.bodyIdA = groundId;
+		weldDef.localAnchorB = b2Vec2_zero;
+
+		float r = 8.0f;
+		for ( float angle = 0.0f; angle < 360.0f; angle += 30.0f )
+		{
+			b2CosSin cosSin = b2ComputeCosSin( angle * B2_PI / 180.0f );
+			bodyDef.position = { r * cosSin.cosine, r * cosSin.sine };
+			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2Polygon box = b2MakeBox( 1.0f, 0.1f );
+			b2CreatePolygonShape( bodyId, &shapeDef, &box );
+
+			weldDef.localAnchorA = bodyDef.position;
+			weldDef.bodyIdB = bodyId;
+			b2JointId jointId = b2CreateWeldJoint( m_worldId, &weldDef );
+			m_jointIds.push_back( jointId );
+		}
+
+		m_radius = 7.0f;
+		m_falloff = 3.0f;
+		m_impulse = 10.0f;
+	}
+
+	void UpdateGui() override
+	{
+		float height = 160.0f;
+		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
+		ImGui::SetNextWindowSize( ImVec2( 240.0f, height ) );
+
+		ImGui::Begin( "Explosion", nullptr, ImGuiWindowFlags_NoMove | ImGuiWindowFlags_NoResize );
+
+		if ( ImGui::Button( "Explode" ) )
+		{
+			b2ExplosionDef def = b2DefaultExplosionDef();
+			def.position = b2Vec2_zero;
+			def.radius = m_radius;
+			def.falloff = m_falloff;
+			def.impulsePerLength = m_impulse;
+			b2World_Explode( m_worldId, &def );
+		}
+
+		ImGui::SliderFloat( "radius", &m_radius, 0.0f, 20.0f, "%.1f" );
+		ImGui::SliderFloat( "falloff", &m_falloff, 0.0f, 20.0f, "%.1f" );
+		ImGui::SliderFloat( "impulse", &m_impulse, -20.0f, 20.0f, "%.1f" );
+
+		ImGui::End();
+	}
+
+	void Step( Settings& settings ) override
+	{
+		if ( settings.pause == false || settings.singleStep == true )
+		{
+			m_referenceAngle += settings.hertz > 0.0f ? 60.0f * B2_PI / 180.0f / settings.hertz : 0.0f;
+			m_referenceAngle = b2UnwindAngle( m_referenceAngle );
+
+			int count = (int)m_jointIds.size();
+			for ( int i = 0; i < count; ++i )
+			{
+				b2WeldJoint_SetReferenceAngle( m_jointIds[i], m_referenceAngle );
+			}
+		}
+
+		Sample::Step( settings );
+
+		g_draw.DrawString( 5, m_textLine, "reference angle = %g", m_referenceAngle );
+		m_textLine += m_textIncrement;
+
+		g_draw.DrawCircle( b2Vec2_zero, m_radius + m_falloff, b2_colorBox2DBlue );
+		g_draw.DrawCircle( b2Vec2_zero, m_radius, b2_colorBox2DYellow );
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new Explosion( settings );
+	}
+
+	std::vector<b2JointId> m_jointIds;
+	float m_radius;
+	float m_falloff;
+	float m_impulse;
+	float m_referenceAngle;
+};
+
+static int sampleExplosion = RegisterSample( "Shapes", "Explosion", Explosion::Create );
+
+// This sample tests a static shape being recreated every step.
+class RecreateStatic : public Sample
+{
+public:
+	explicit RecreateStatic( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 0.0f, 2.5f };
+			g_camera.m_zoom = 3.5f;
+		}
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		bodyDef.type = b2_dynamicBody;
+		bodyDef.position = { 0.0f, 1.0f };
+		b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+		b2Polygon box = b2MakeBox( 1.0f, 1.0f );
+		b2CreatePolygonShape( bodyId, &shapeDef, &box );
+
+		m_groundId = {};
+	}
+
+	void Step( Settings& settings ) override
+	{
+		if ( B2_IS_NON_NULL( m_groundId ) )
+		{
+			b2DestroyBody( m_groundId );
+			m_groundId = {};
+		}
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		m_groundId = b2CreateBody( m_worldId, &bodyDef );
+
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+
+		// Invoke contact creation so that contact points are created immediately
+		// on a static body.
+		shapeDef.invokeContactCreation = true;
+
+		b2Segment segment = { { -10.0f, 0.0f }, { 10.0f, 0.0f } };
+		b2CreateSegmentShape( m_groundId, &shapeDef, &segment );
+
+		Sample::Step( settings );
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new RecreateStatic( settings );
+	}
+
+	b2BodyId m_groundId;
+};
+
+static int sampleSingleBox = RegisterSample( "Shapes", "Recreate Static", RecreateStatic::Create );
