@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: MIT
 
 #include "draw.h"
+#include "random.h"
 #include "sample.h"
 #include "settings.h"
 
@@ -10,6 +11,7 @@
 
 #include <GLFW/glfw3.h>
 #include <imgui.h>
+#include <vector>
 
 class SingleBox : public Sample
 {
@@ -30,16 +32,17 @@ public:
 
 		float groundWidth = 66.0f * extent;
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
-		shapeDef.friction = 0.5f;
+		// shapeDef.friction = 0.5f;
 
 		b2Segment segment = { { -0.5f * 2.0f * groundWidth, 0.0f }, { 0.5f * 2.0f * groundWidth, 0.0f } };
 		b2CreateSegmentShape( groundId, &shapeDef, &segment );
 		bodyDef.type = b2_dynamicBody;
 
 		b2Polygon box = b2MakeBox( extent, extent );
-		bodyDef.position = { 0.0f, 4.0f };
-		b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
-		b2CreatePolygonShape( bodyId, &shapeDef, &box );
+		bodyDef.position = { 0.0f, 1.0f };
+		bodyDef.linearVelocity = { 5.0f, 0.0f };
+		m_bodyId = b2CreateBody( m_worldId, &bodyDef );
+		b2CreatePolygonShape( m_bodyId, &shapeDef, &box );
 	}
 
 	void Step( Settings& settings ) override
@@ -47,12 +50,17 @@ public:
 		Sample::Step( settings );
 
 		// g_draw.DrawCircle({0.0f, 2.0f}, 1.0f, b2_colorWhite);
+
+		b2Vec2 position = b2Body_GetPosition( m_bodyId );
+		DrawTextLine( "(x, y) = (%.2g, %.2g)", position.x, position.y );
 	}
 
 	static Sample* Create( Settings& settings )
 	{
 		return new SingleBox( settings );
 	}
+
+	b2BodyId m_bodyId;
 };
 
 static int sampleSingleBox = RegisterSample( "Stacking", "Single Box", SingleBox::Create );
@@ -60,12 +68,6 @@ static int sampleSingleBox = RegisterSample( "Stacking", "Single Box", SingleBox
 class TiltedStack : public Sample
 {
 public:
-	enum
-	{
-		e_columns = 10,
-		e_rows = 10,
-	};
-
 	explicit TiltedStack( Settings& settings )
 		: Sample( settings )
 	{
@@ -85,7 +87,7 @@ public:
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 		}
 
-		for ( int i = 0; i < e_rows * e_columns; ++i )
+		for ( int i = 0; i < m_rows * m_columns; ++i )
 		{
 			m_bodies[i] = b2_nullBodyId;
 		}
@@ -94,22 +96,22 @@ public:
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
 		shapeDef.density = 1.0f;
-		shapeDef.friction = 0.3f;
+		shapeDef.material.friction = 0.3f;
 
 		float offset = 0.2f;
 		float dx = 5.0f;
-		float xroot = -0.5f * dx * ( e_columns - 1.0f );
+		float xroot = -0.5f * dx * ( m_columns - 1.0f );
 
-		for ( int j = 0; j < e_columns; ++j )
+		for ( int j = 0; j < m_columns; ++j )
 		{
 			float x = xroot + j * dx;
 
-			for ( int i = 0; i < e_rows; ++i )
+			for ( int i = 0; i < m_rows; ++i )
 			{
 				b2BodyDef bodyDef = b2DefaultBodyDef();
 				bodyDef.type = b2_dynamicBody;
 
-				int n = j * e_rows + i;
+				int n = j * m_rows + i;
 
 				bodyDef.position = { x + offset * i, 0.5f + 1.0f * i };
 				b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
@@ -126,18 +128,26 @@ public:
 		return new TiltedStack( settings );
 	}
 
-	b2BodyId m_bodies[e_rows * e_columns];
+	static constexpr int m_columns = 10;
+	static constexpr int m_rows = 10;
+
+	b2BodyId m_bodies[m_rows * m_columns];
 };
 
 static int sampleTiltedStack = RegisterSample( "Stacking", "Tilted Stack", TiltedStack::Create );
 
+// This sample shows some aspects of Box2D continuous collision:
+// - bullet dynamic bodies which support continuous collision with non-bullet dynamic bodies
+// - prevention of chain reaction tunneling
+// Try disabling continuous collision and firing a bullet. You might see a bullet push a
+// a through the static wall.
 class VerticalStack : public Sample
 {
 public:
 	enum
 	{
 		e_maxColumns = 10,
-		e_maxRows = 12,
+		e_maxRows = 15,
 		e_maxBullets = 8
 	};
 
@@ -158,14 +168,15 @@ public:
 
 		{
 			b2BodyDef bodyDef = b2DefaultBodyDef();
-			bodyDef.position = { 0.0f, -1.0f };
+			bodyDef.position = { 0.0f, 0.0f };
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
-			b2Polygon box = b2MakeBox( 100.0f, 1.0f );
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
-			b2Segment segment = { { 10.0f, 1.0f }, { 10.0f, 21.0f } };
+			b2Segment segment = { { 10.0f, 0.0f }, { 10.0f, 20.0f } };
+			b2CreateSegmentShape( groundId, &shapeDef, &segment );
+
+			segment = { { -30.0f, 0.0f }, { 30.0f, 0.0f } };
 			b2CreateSegmentShape( groundId, &shapeDef, &segment );
 		}
 
@@ -180,8 +191,8 @@ public:
 		}
 
 		m_shapeType = e_boxShape;
-		m_rowCount = e_maxRows;
-		m_columnCount = 5;
+		m_rowCount = 12;
+		m_columnCount = 1;
 		m_bulletCount = 1;
 		m_bulletType = e_circleShape;
 
@@ -199,15 +210,14 @@ public:
 			}
 		}
 
-		b2Circle circle = { 0 };
+		b2Circle circle = {};
 		circle.radius = 0.5f;
 
-		b2Polygon box = b2MakeBox( 0.5f, 0.5f );
-		// b2Polygon box = b2MakeRoundedBox(0.45f, 0.45f, 0.05f);
+		b2Polygon box = b2MakeRoundedBox( 0.45f, 0.45f, 0.05f );
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
 		shapeDef.density = 1.0f;
-		shapeDef.friction = 0.3f;
+		shapeDef.material.friction = 0.3f;
 
 		float offset;
 
@@ -297,8 +307,8 @@ public:
 		{
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			bodyDef.type = b2_dynamicBody;
-			bodyDef.position = { -25.0f - i, 6.0f };
-			float speed = RandomFloat( 200.0f, 300.0f );
+			bodyDef.position = { -26.7f - i, 6.0f };
+			float speed = RandomFloatRange( 200.0f, 300.0f );
 			bodyDef.linearVelocity = { speed, 0.0f };
 			bodyDef.isBullet = true;
 
@@ -317,7 +327,7 @@ public:
 		}
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 230.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -383,24 +393,34 @@ public:
 
 static int sampleVerticalStack = RegisterSample( "Stacking", "Vertical Stack", VerticalStack::Create );
 
-// This shows how to handle high gravity and small shapes using a small time step
+// A simple circle stack that also shows how to collect hit events
 class CircleStack : public Sample
 {
 public:
+	struct Event
+	{
+		int indexA, indexB;
+	};
+
 	explicit CircleStack( Settings& settings )
 		: Sample( settings )
 	{
 		if ( settings.restart == false )
 		{
-			g_camera.m_center = { 0.0f, 2.0f };
-			g_camera.m_zoom = 3.0f;
+			g_camera.m_center = { 0.0f, 5.0f };
+			g_camera.m_zoom = 6.0f;
 		}
 
+		int shapeIndex = 0;
+
 		{
 			b2BodyDef bodyDef = b2DefaultBodyDef();
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			shapeDef.userData = reinterpret_cast<void*>( intptr_t( shapeIndex ) );
+			shapeIndex += 1;
+
 			b2Segment segment = { { -10.0f, 0.0f }, { 10.0f, 0.0f } };
 			b2CreateSegmentShape( groundId, &shapeDef, &segment );
 		}
@@ -408,23 +428,58 @@ public:
 		b2World_SetGravity( m_worldId, { 0.0f, -20.0f } );
 		b2World_SetContactTuning( m_worldId, 0.25f * 360.0f, 10.0f, 3.0f );
 
-		b2Circle circle = {};
-		circle.radius = 0.1f;
-
-		b2ShapeDef shapeDef = b2DefaultShapeDef();
 		b2BodyDef bodyDef = b2DefaultBodyDef();
 		bodyDef.type = b2_dynamicBody;
 
-		float y = 0.5f;
+		b2Circle circle = {};
+		circle.radius = 0.5f;
 
-		for ( int i = 0; i < 20; ++i )
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		shapeDef.enableHitEvents = true;
+		// shapeDef.rollingResistance = 0.2f;
+		shapeDef.material.friction = 0.0f;
+
+		float y = 0.75f;
+
+		for ( int i = 0; i < 10; ++i )
 		{
 			bodyDef.position.y = y;
 
 			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			shapeDef.userData = reinterpret_cast<void*>( intptr_t( shapeIndex ) );
+			shapeDef.density = 1.0f + 4.0f * i;
+			shapeIndex += 1;
 			b2CreateCircleShape( bodyId, &shapeDef, &circle );
 
-			y += 1.0f;
+			y += 1.25f;
+		}
+	}
+
+	void Step( Settings& settings ) override
+	{
+		Sample::Step( settings );
+
+		b2ContactEvents events = b2World_GetContactEvents( m_worldId );
+		for ( int i = 0; i < events.hitCount; ++i )
+		{
+			b2ContactHitEvent* event = events.hitEvents + i;
+
+			void* userDataA = b2Shape_GetUserData( event->shapeIdA );
+			void* userDataB = b2Shape_GetUserData( event->shapeIdB );
+			int indexA = static_cast<int>( reinterpret_cast<intptr_t>( userDataA ) );
+			int indexB = static_cast<int>( reinterpret_cast<intptr_t>( userDataB ) );
+
+			g_draw.DrawPoint( event->point, 10.0f, b2_colorWhite );
+
+			m_events.push_back( { indexA, indexB } );
+		}
+
+		int eventCount = (int)m_events.size();
+		for ( int i = 0; i < eventCount; ++i )
+		{
+			g_draw.DrawString( 5, m_textLine, "%d, %d", m_events[i].indexA, m_events[i].indexB );
+			m_textLine += m_textIncrement;
 		}
 	}
 
@@ -432,10 +487,73 @@ public:
 	{
 		return new CircleStack( settings );
 	}
+
+	std::vector<Event> m_events;
 };
 
 static int sampleCircleStack = RegisterSample( "Stacking", "Circle Stack", CircleStack::Create );
 
+class CapsuleStack : public Sample
+{
+public:
+	struct Event
+	{
+		int indexA, indexB;
+	};
+
+	explicit CapsuleStack( Settings& settings )
+		: Sample( settings )
+	{
+		if ( settings.restart == false )
+		{
+			g_camera.m_center = { 0.0f, 5.0f };
+			g_camera.m_zoom = 6.0f;
+		}
+
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.position = { 0.0f, -1.0f };
+			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2Polygon polygon = b2MakeBox( 10.0f, 1.0f );
+			b2CreatePolygonShape( groundId, &shapeDef, &polygon );
+		}
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		bodyDef.type = b2_dynamicBody;
+
+		float a = 0.25f;
+		b2Capsule capsule = { { -4.0f * a, 0.0f }, { 4.0f * a, 0.0f }, a };
+
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+
+		// rolling resistance increases stacking stability
+		// shapeDef.rollingResistance = 0.2f;
+
+		float y = 2.0f * a;
+
+		for ( int i = 0; i < 20; ++i )
+		{
+			bodyDef.position.y = y;
+			// bodyDef.position.x += ( i & 1 ) == 1 ? -0.5f * a : 0.5f * a;
+			// bodyDef.linearVelocity = { 0.0f, -10.0f };
+			b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2CreateCapsuleShape( bodyId, &shapeDef, &capsule );
+
+			y += 3.0f * a;
+		}
+	}
+
+	static Sample* Create( Settings& settings )
+	{
+		return new CapsuleStack( settings );
+	}
+};
+
+static int sampleCapsuleStack = RegisterSample( "Stacking", "Capsule Stack", CapsuleStack::Create );
+
 class Cliff : public Sample
 {
 public:
@@ -454,13 +572,13 @@ public:
 			b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			b2Polygon box = b2MakeOffsetBox( 100.0f, 1.0f, { 0.0f, -1.0f }, 0.0f );
+			b2Polygon box = b2MakeOffsetBox( 100.0f, 1.0f, { 0.0f, -1.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
 			b2Segment segment = { { -14.0f, 4.0f }, { -8.0f, 4.0f } };
 			b2CreateSegmentShape( groundId, &shapeDef, &segment );
 
-			box = b2MakeOffsetBox( 3.0f, 0.5f, { 0.0f, 4.0f }, 0.0f );
+			box = b2MakeOffsetBox( 3.0f, 0.5f, { 0.0f, 4.0f }, b2Rot_identity );
 			b2CreatePolygonShape( groundId, &shapeDef, &box );
 
 			b2Capsule capsule = { { 8.5f, 4.0f }, { 13.5f, 4.0f }, 0.5f };
@@ -499,7 +617,7 @@ public:
 
 		{
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.01f;
+			shapeDef.material.friction = 0.01f;
 			bodyDef.linearVelocity = { 2.0f * sign, 0.0f };
 
 			float offset = m_flip ? -4.0f : 0.0f;
@@ -519,7 +637,7 @@ public:
 
 		{
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.01f;
+			shapeDef.material.friction = 0.01f;
 			bodyDef.linearVelocity = { 2.5f * sign, 0.0f };
 
 			bodyDef.position = { -11.0f, 4.5f };
@@ -537,7 +655,7 @@ public:
 
 		{
 			b2ShapeDef shapeDef = b2DefaultShapeDef();
-			shapeDef.friction = 0.2f;
+			shapeDef.material.friction = 0.2f;
 			bodyDef.linearVelocity = { 1.5f * sign, 0.0f };
 
 			float offset = m_flip ? 4.0f : 0.0f;
@@ -556,7 +674,7 @@ public:
 		}
 	}
 
-	void UpdateUI() override
+	void UpdateGui() override
 	{
 		float height = 60.0f;
 		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
@@ -624,7 +742,7 @@ public:
 		}
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
-		shapeDef.friction = 0.6f;
+		shapeDef.material.friction = 0.6f;
 
 		{
 			b2BodyDef bodyDef = b2DefaultBodyDef();
@@ -707,7 +825,7 @@ public:
 		b2Polygon box = b2MakeBox( 0.125f, 0.5f );
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
-		shapeDef.friction = 0.6f;
+		shapeDef.material.friction = 0.6f;
 		b2BodyDef bodyDef = b2DefaultBodyDef();
 		bodyDef.type = b2_dynamicBody;
 
@@ -738,12 +856,6 @@ static int sampleDoubleDomino = RegisterSample( "Stacking", "Double Domino", Dou
 class Confined : public Sample
 {
 public:
-	enum
-	{
-		e_gridCount = 25,
-		e_maxCount = e_gridCount * e_gridCount
-	};
-
 	explicit Confined( Settings& settings )
 		: Sample( settings )
 	{
@@ -780,13 +892,13 @@ public:
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
 		b2Circle circle = { { 0.0f, 0.0f }, 0.5f };
 
-		while ( m_count < e_maxCount )
+		while ( m_count < m_maxCount )
 		{
 			m_row = 0;
-			for ( int i = 0; i < e_gridCount; ++i )
+			for ( int i = 0; i < m_gridCount; ++i )
 			{
-				float x = -8.75f + m_column * 18.0f / e_gridCount;
-				float y = 1.5f + m_row * 18.0f / e_gridCount;
+				float x = -8.75f + m_column * 18.0f / m_gridCount;
+				float y = 1.5f + m_row * 18.0f / m_gridCount;
 
 				bodyDef.position = { x, y };
 				b2BodyId bodyId = b2CreateBody( m_worldId, &bodyDef );
@@ -804,6 +916,8 @@ public:
 		return new Confined( settings );
 	}
 
+	static constexpr int m_gridCount = 25;
+	static constexpr int m_maxCount = m_gridCount * m_gridCount;
 	int m_row;
 	int m_column;
 	int m_count;
@@ -829,7 +943,7 @@ public:
 		b2BodyId groundId = b2CreateBody( m_worldId, &bodyDef );
 
 		b2ShapeDef shapeDef = b2DefaultShapeDef();
-		shapeDef.friction = 0.7f;
+		shapeDef.material.friction = 0.7f;
 
 		b2Polygon groundBox = b2MakeBox( 40.0f, 2.0f );
 		b2CreatePolygonShape( groundId, &shapeDef, &groundBox );
@@ -837,9 +951,9 @@ public:
 		float cardHeight = 0.2f;
 		float cardThickness = 0.001f;
 
-		float angle0 = 25.0f * b2_pi / 180.0f;
-		float angle1 = -25.0f * b2_pi / 180.0f;
-		float angle2 = 0.5f * b2_pi;
+		float angle0 = 25.0f * B2_PI / 180.0f;
+		float angle1 = -25.0f * B2_PI / 180.0f;
+		float angle2 = 0.5f * B2_PI;
 
 		b2Polygon cardBox = b2MakeBox( cardThickness, cardHeight );
 		bodyDef.type = b2_dynamicBody;
