@@ -10,27 +10,28 @@ struct Settings
 	void Load();
 
 	int sampleIndex = 0;
-	// int windowWidth = 3840;
-	// int windowHeight = 2160;
 	int windowWidth = 1920;
 	int windowHeight = 1080;
-	// int windowWidth = 1280;
-	// int windowHeight = 720;
-	// int windowWidth = 800;
-	// int windowHeight = 600;
+	//int windowWidth = 900;
+	//int windowHeight = 600;
+
 	float hertz = 60.0f;
 	int subStepCount = 4;
 	int workerCount = 1;
+
 	bool useCameraBounds = false;
 	bool drawShapes = true;
 	bool drawJoints = true;
 	bool drawJointExtras = false;
-	bool drawAABBs = false;
+	bool drawBounds = false;
+	bool drawMass = false;
+	bool drawBodyNames = false;
 	bool drawContactPoints = false;
 	bool drawContactNormals = false;
 	bool drawContactImpulses = false;
+	bool drawContactFeatures = false;
 	bool drawFrictionImpulses = false;
-	bool drawMass = false;
+	bool drawIslands = false;
 	bool drawGraphColors = false;
 	bool drawCounters = false;
 	bool drawProfile = false;
