@@ -1,8 +1,7 @@
 // SPDX-FileCopyrightText: 2023 Erin Catto
 // SPDX-License-Identifier: MIT
 
-#include "aabb.h"
-#include "core.h"
+#include "constants.h"
 #include "shape.h"
 
 #include "box2d/collision.h"
@@ -11,16 +10,16 @@
 #include <float.h>
 #include <stddef.h>
 
-_Static_assert( b2_maxPolygonVertices > 2, "must be 3 or more" );
+_Static_assert( B2_MAX_POLYGON_VERTICES > 2, "must be 3 or more" );
 
 bool b2IsValidRay( const b2RayCastInput* input )
 {
-	bool isValid = b2Vec2_IsValid( input->origin ) && b2Vec2_IsValid( input->translation ) && b2IsValid( input->maxFraction ) &&
-				   0.0f <= input->maxFraction && input->maxFraction < b2_huge;
+	bool isValid = b2IsValidVec2( input->origin ) && b2IsValidVec2( input->translation ) &&
+				   b2IsValidFloat( input->maxFraction ) && 0.0f <= input->maxFraction && input->maxFraction < B2_HUGE;
 	return isValid;
 }
 
-static b2Vec2 b2ComputePolygonCentroid( const b2Vec2* vertices, int32_t count )
+static b2Vec2 b2ComputePolygonCentroid( const b2Vec2* vertices, int count )
 {
 	b2Vec2 center = { 0.0f, 0.0f };
 	float area = 0.0f;
@@ -31,7 +30,7 @@ static b2Vec2 b2ComputePolygonCentroid( const b2Vec2* vertices, int32_t count )
 
 	const float inv3 = 1.0f / 3.0f;
 
-	for ( int32_t i = 1; i < count - 1; ++i )
+	for ( int i = 1; i < count - 1; ++i )
 	{
 		// Triangle edges
 		b2Vec2 e1 = b2Sub( vertices[i], origin );
@@ -69,16 +68,16 @@ b2Polygon b2MakePolygon( const b2Hull* hull, float radius )
 	shape.radius = radius;
 
 	// Copy vertices
-	for ( int32_t i = 0; i < shape.count; ++i )
+	for ( int i = 0; i < shape.count; ++i )
 	{
 		shape.vertices[i] = hull->points[i];
 	}
 
 	// Compute normals. Ensure the edges have non-zero length.
-	for ( int32_t i = 0; i < shape.count; ++i )
+	for ( int i = 0; i < shape.count; ++i )
 	{
-		int32_t i1 = i;
-		int32_t i2 = i + 1 < shape.count ? i + 1 : 0;
+		int i1 = i;
+		int i2 = i + 1 < shape.count ? i + 1 : 0;
 		b2Vec2 edge = b2Sub( shape.vertices[i2], shape.vertices[i1] );
 		B2_ASSERT( b2Dot( edge, edge ) > FLT_EPSILON * FLT_EPSILON );
 		shape.normals[i] = b2Normalize( b2CrossVS( edge, 1.0f ) );
@@ -89,7 +88,12 @@ b2Polygon b2MakePolygon( const b2Hull* hull, float radius )
 	return shape;
 }
 
-b2Polygon b2MakeOffsetPolygon( const b2Hull* hull, float radius, b2Transform transform )
+b2Polygon b2MakeOffsetPolygon( const b2Hull* hull, b2Vec2 position, b2Rot rotation )
+{
+	return b2MakeOffsetRoundedPolygon( hull, position, rotation, 0.0f );
+}
+
+b2Polygon b2MakeOffsetRoundedPolygon( const b2Hull* hull, b2Vec2 position, b2Rot rotation, float radius )
 {
 	B2_ASSERT( b2ValidateHull( hull ) );
 
@@ -99,21 +103,23 @@ b2Polygon b2MakeOffsetPolygon( const b2Hull* hull, float radius, b2Transform tra
 		return b2MakeSquare( 0.5f );
 	}
 
+	b2Transform transform = { position, rotation };
+
 	b2Polygon shape = { 0 };
 	shape.count = hull->count;
 	shape.radius = radius;
 
 	// Copy vertices
-	for ( int32_t i = 0; i < shape.count; ++i )
+	for ( int i = 0; i < shape.count; ++i )
 	{
 		shape.vertices[i] = b2TransformPoint( transform, hull->points[i] );
 	}
 
 	// Compute normals. Ensure the edges have non-zero length.
-	for ( int32_t i = 0; i < shape.count; ++i )
+	for ( int i = 0; i < shape.count; ++i )
 	{
-		int32_t i1 = i;
-		int32_t i2 = i + 1 < shape.count ? i + 1 : 0;
+		int i1 = i;
+		int i2 = i + 1 < shape.count ? i + 1 : 0;
 		b2Vec2 edge = b2Sub( shape.vertices[i2], shape.vertices[i1] );
 		B2_ASSERT( b2Dot( edge, edge ) > FLT_EPSILON * FLT_EPSILON );
 		shape.normals[i] = b2Normalize( b2CrossVS( edge, 1.0f ) );
@@ -124,56 +130,75 @@ b2Polygon b2MakeOffsetPolygon( const b2Hull* hull, float radius, b2Transform tra
 	return shape;
 }
 
-b2Polygon b2MakeSquare( float h )
+b2Polygon b2MakeSquare( float halfWidth )
 {
-	return b2MakeBox( h, h );
+	return b2MakeBox( halfWidth, halfWidth );
 }
 
-b2Polygon b2MakeBox( float hx, float hy )
+b2Polygon b2MakeBox( float halfWidth, float halfHeight )
 {
-	B2_ASSERT( b2IsValid( hx ) && hx > 0.0f );
-	B2_ASSERT( b2IsValid( hy ) && hy > 0.0f );
+	B2_ASSERT( b2IsValidFloat( halfWidth ) && halfWidth > 0.0f );
+	B2_ASSERT( b2IsValidFloat( halfHeight ) && halfHeight > 0.0f );
 
 	b2Polygon shape = { 0 };
 	shape.count = 4;
-	shape.vertices[0] = ( b2Vec2 ){ -hx, -hy };
-	shape.vertices[1] = ( b2Vec2 ){ hx, -hy };
-	shape.vertices[2] = ( b2Vec2 ){ hx, hy };
-	shape.vertices[3] = ( b2Vec2 ){ -hx, hy };
-	shape.normals[0] = ( b2Vec2 ){ 0.0f, -1.0f };
-	shape.normals[1] = ( b2Vec2 ){ 1.0f, 0.0f };
-	shape.normals[2] = ( b2Vec2 ){ 0.0f, 1.0f };
-	shape.normals[3] = ( b2Vec2 ){ -1.0f, 0.0f };
+	shape.vertices[0] = (b2Vec2){ -halfWidth, -halfHeight };
+	shape.vertices[1] = (b2Vec2){ halfWidth, -halfHeight };
+	shape.vertices[2] = (b2Vec2){ halfWidth, halfHeight };
+	shape.vertices[3] = (b2Vec2){ -halfWidth, halfHeight };
+	shape.normals[0] = (b2Vec2){ 0.0f, -1.0f };
+	shape.normals[1] = (b2Vec2){ 1.0f, 0.0f };
+	shape.normals[2] = (b2Vec2){ 0.0f, 1.0f };
+	shape.normals[3] = (b2Vec2){ -1.0f, 0.0f };
 	shape.radius = 0.0f;
 	shape.centroid = b2Vec2_zero;
 	return shape;
 }
 
-b2Polygon b2MakeRoundedBox( float hx, float hy, float radius )
+b2Polygon b2MakeRoundedBox( float halfWidth, float halfHeight, float radius )
 {
-	b2Polygon shape = b2MakeBox( hx, hy );
+	B2_ASSERT( b2IsValidFloat( radius ) && radius >= 0.0f );
+	b2Polygon shape = b2MakeBox( halfWidth, halfHeight );
 	shape.radius = radius;
 	return shape;
 }
 
-b2Polygon b2MakeOffsetBox( float hx, float hy, b2Vec2 center, float angle )
+b2Polygon b2MakeOffsetBox( float halfWidth, float halfHeight, b2Vec2 center, b2Rot rotation )
 {
-	b2Transform xf;
-	xf.p = center;
-	xf.q = b2MakeRot( angle );
+	b2Transform xf = { center, rotation };
 
 	b2Polygon shape = { 0 };
 	shape.count = 4;
-	shape.vertices[0] = b2TransformPoint( xf, ( b2Vec2 ){ -hx, -hy } );
-	shape.vertices[1] = b2TransformPoint( xf, ( b2Vec2 ){ hx, -hy } );
-	shape.vertices[2] = b2TransformPoint( xf, ( b2Vec2 ){ hx, hy } );
-	shape.vertices[3] = b2TransformPoint( xf, ( b2Vec2 ){ -hx, hy } );
-	shape.normals[0] = b2RotateVector( xf.q, ( b2Vec2 ){ 0.0f, -1.0f } );
-	shape.normals[1] = b2RotateVector( xf.q, ( b2Vec2 ){ 1.0f, 0.0f } );
-	shape.normals[2] = b2RotateVector( xf.q, ( b2Vec2 ){ 0.0f, 1.0f } );
-	shape.normals[3] = b2RotateVector( xf.q, ( b2Vec2 ){ -1.0f, 0.0f } );
+	shape.vertices[0] = b2TransformPoint( xf, (b2Vec2){ -halfWidth, -halfHeight } );
+	shape.vertices[1] = b2TransformPoint( xf, (b2Vec2){ halfWidth, -halfHeight } );
+	shape.vertices[2] = b2TransformPoint( xf, (b2Vec2){ halfWidth, halfHeight } );
+	shape.vertices[3] = b2TransformPoint( xf, (b2Vec2){ -halfWidth, halfHeight } );
+	shape.normals[0] = b2RotateVector( xf.q, (b2Vec2){ 0.0f, -1.0f } );
+	shape.normals[1] = b2RotateVector( xf.q, (b2Vec2){ 1.0f, 0.0f } );
+	shape.normals[2] = b2RotateVector( xf.q, (b2Vec2){ 0.0f, 1.0f } );
+	shape.normals[3] = b2RotateVector( xf.q, (b2Vec2){ -1.0f, 0.0f } );
 	shape.radius = 0.0f;
-	shape.centroid = center;
+	shape.centroid = xf.p;
+	return shape;
+}
+
+b2Polygon b2MakeOffsetRoundedBox( float halfWidth, float halfHeight, b2Vec2 center, b2Rot rotation, float radius )
+{
+	B2_ASSERT( b2IsValidFloat( radius ) && radius >= 0.0f );
+	b2Transform xf = { center, rotation };
+
+	b2Polygon shape = { 0 };
+	shape.count = 4;
+	shape.vertices[0] = b2TransformPoint( xf, (b2Vec2){ -halfWidth, -halfHeight } );
+	shape.vertices[1] = b2TransformPoint( xf, (b2Vec2){ halfWidth, -halfHeight } );
+	shape.vertices[2] = b2TransformPoint( xf, (b2Vec2){ halfWidth, halfHeight } );
+	shape.vertices[3] = b2TransformPoint( xf, (b2Vec2){ -halfWidth, halfHeight } );
+	shape.normals[0] = b2RotateVector( xf.q, (b2Vec2){ 0.0f, -1.0f } );
+	shape.normals[1] = b2RotateVector( xf.q, (b2Vec2){ 1.0f, 0.0f } );
+	shape.normals[2] = b2RotateVector( xf.q, (b2Vec2){ 0.0f, 1.0f } );
+	shape.normals[3] = b2RotateVector( xf.q, (b2Vec2){ -1.0f, 0.0f } );
+	shape.radius = radius;
+	shape.centroid = xf.p;
 	return shape;
 }
 
@@ -197,7 +222,7 @@ b2MassData b2ComputeCircleMass( const b2Circle* shape, float density )
 	float rr = shape->radius * shape->radius;
 
 	b2MassData massData;
-	massData.mass = density * b2_pi * rr;
+	massData.mass = density * B2_PI * rr;
 	massData.center = shape->center;
 
 	// inertia about the local origin
@@ -215,7 +240,7 @@ b2MassData b2ComputeCapsuleMass( const b2Capsule* shape, float density )
 	float length = b2Length( b2Sub( p2, p1 ) );
 	float ll = length * length;
 
-	float circleMass = density * ( b2_pi * radius * radius );
+	float circleMass = density * ( B2_PI * radius * radius );
 	float boxMass = density * ( 2.0f * radius * length );
 
 	b2MassData massData;
@@ -233,7 +258,7 @@ b2MassData b2ComputeCapsuleMass( const b2Capsule* shape, float density )
 	// I verified this formula by computing the convex hull of a 128 vertex capsule
 
 	// half circle centroid
-	float lc = 4.0f * radius / ( 3.0f * b2_pi );
+	float lc = 4.0f * radius / ( 3.0f * B2_PI );
 
 	// half length of rectangular portion of capsule
 	float h = 0.5f * length;
@@ -293,17 +318,17 @@ b2MassData b2ComputePolygonMass( const b2Polygon* shape, float density )
 		return b2ComputeCapsuleMass( &capsule, density );
 	}
 
-	b2Vec2 vertices[b2_maxPolygonVertices] = { 0 };
-	int32_t count = shape->count;
+	b2Vec2 vertices[B2_MAX_POLYGON_VERTICES] = { 0 };
+	int count = shape->count;
 	float radius = shape->radius;
 
 	if ( radius > 0.0f )
 	{
 		// Approximate mass of rounded polygons by pushing out the vertices.
 		float sqrt2 = 1.412f;
-		for ( int32_t i = 0; i < count; ++i )
+		for ( int i = 0; i < count; ++i )
 		{
-			int32_t j = i == 0 ? count - 1 : i - 1;
+			int j = i == 0 ? count - 1 : i - 1;
 			b2Vec2 n1 = shape->normals[j];
 			b2Vec2 n2 = shape->normals[i];
 
@@ -313,7 +338,7 @@ b2MassData b2ComputePolygonMass( const b2Polygon* shape, float density )
 	}
 	else
 	{
-		for ( int32_t i = 0; i < count; ++i )
+		for ( int i = 0; i < count; ++i )
 		{
 			vertices[i] = shape->vertices[i];
 		}
@@ -329,7 +354,7 @@ b2MassData b2ComputePolygonMass( const b2Polygon* shape, float density )
 
 	const float inv3 = 1.0f / 3.0f;
 
-	for ( int32_t i = 1; i < count - 1; ++i )
+	for ( int i = 1; i < count - 1; ++i )
 	{
 		// Triangle edges
 		b2Vec2 e1 = b2Sub( vertices[i], r );
@@ -401,7 +426,7 @@ b2AABB b2ComputePolygonAABB( const b2Polygon* shape, b2Transform xf )
 	b2Vec2 lower = b2TransformPoint( xf, shape->vertices[0] );
 	b2Vec2 upper = lower;
 
-	for ( int32_t i = 1; i < shape->count; ++i )
+	for ( int i = 1; i < shape->count; ++i )
 	{
 		b2Vec2 v = b2TransformPoint( xf, shape->vertices[i] );
 		lower = b2Min( lower, v );
@@ -470,8 +495,8 @@ bool b2PointInPolygon( b2Vec2 point, const b2Polygon* shape )
 	input.transformB = b2Transform_identity;
 	input.useRadii = false;
 
-	b2DistanceCache cache = { 0 };
-	b2DistanceOutput output = b2ShapeDistance( &cache, &input, NULL, 0 );
+	b2SimplexCache cache = { 0 };
+	b2DistanceOutput output = b2ShapeDistance( &input, &cache, NULL, 0 );
 
 	return output.distance <= shape->radius;
 }
@@ -514,7 +539,7 @@ b2CastOutput b2RayCastCircle( const b2RayCastInput* input, const b2Circle* shape
 		return output;
 	}
 
-	// Pythagorus
+	// Pythagoras
 	float h = sqrtf( rr - cc );
 
 	float fraction = t - h;
@@ -525,6 +550,7 @@ b2CastOutput b2RayCastCircle( const b2RayCastInput* input, const b2Circle* shape
 		return output;
 	}
 
+	// hit point relative to center
 	b2Vec2 hitPoint = b2MulAdd( s, fraction, d );
 
 	output.fraction = fraction / length;
@@ -743,7 +769,7 @@ b2CastOutput b2RayCastSegment( const b2RayCastInput* input, const b2Segment* sha
 	}
 
 	output.fraction = t;
-	output.point = b2MulAdd( p1, t, d );
+	output.point = p;
 	output.normal = normal;
 	output.hit = true;
 
@@ -762,11 +788,11 @@ b2CastOutput b2RayCastPolygon( const b2RayCastInput* input, const b2Polygon* sha
 
 		float lower = 0.0f, upper = input->maxFraction;
 
-		int32_t index = -1;
+		int index = -1;
 
 		b2CastOutput output = { 0 };
 
-		for ( int32_t i = 0; i < shape->count; ++i )
+		for ( int i = 0; i < shape->count; ++i )
 		{
 			// p = p1 + a * d
 			// dot(normal, p - v) = 0
@@ -833,6 +859,7 @@ b2CastOutput b2RayCastPolygon( const b2RayCastInput* input, const b2Polygon* sha
 	castInput.transformB = b2Transform_identity;
 	castInput.translationB = input->translation;
 	castInput.maxFraction = input->maxFraction;
+	castInput.canEncroach = false;
 	return b2ShapeCast( &castInput );
 }
 
@@ -840,11 +867,12 @@ b2CastOutput b2ShapeCastCircle( const b2ShapeCastInput* input, const b2Circle* s
 {
 	b2ShapeCastPairInput pairInput;
 	pairInput.proxyA = b2MakeProxy( &shape->center, 1, shape->radius );
-	pairInput.proxyB = b2MakeProxy( input->points, input->count, input->radius );
+	pairInput.proxyB = input->proxy;
 	pairInput.transformA = b2Transform_identity;
 	pairInput.transformB = b2Transform_identity;
 	pairInput.translationB = input->translation;
 	pairInput.maxFraction = input->maxFraction;
+	pairInput.canEncroach = input->canEncroach;
 
 	b2CastOutput output = b2ShapeCast( &pairInput );
 	return output;
@@ -854,11 +882,12 @@ b2CastOutput b2ShapeCastCapsule( const b2ShapeCastInput* input, const b2Capsule*
 {
 	b2ShapeCastPairInput pairInput;
 	pairInput.proxyA = b2MakeProxy( &shape->center1, 2, shape->radius );
-	pairInput.proxyB = b2MakeProxy( input->points, input->count, input->radius );
+	pairInput.proxyB = input->proxy;
 	pairInput.transformA = b2Transform_identity;
 	pairInput.transformB = b2Transform_identity;
 	pairInput.translationB = input->translation;
 	pairInput.maxFraction = input->maxFraction;
+	pairInput.canEncroach = input->canEncroach;
 
 	b2CastOutput output = b2ShapeCast( &pairInput );
 	return output;
@@ -868,11 +897,12 @@ b2CastOutput b2ShapeCastSegment( const b2ShapeCastInput* input, const b2Segment*
 {
 	b2ShapeCastPairInput pairInput;
 	pairInput.proxyA = b2MakeProxy( &shape->point1, 2, 0.0f );
-	pairInput.proxyB = b2MakeProxy( input->points, input->count, input->radius );
+	pairInput.proxyB = input->proxy;
 	pairInput.transformA = b2Transform_identity;
 	pairInput.transformB = b2Transform_identity;
 	pairInput.translationB = input->translation;
 	pairInput.maxFraction = input->maxFraction;
+	pairInput.canEncroach = input->canEncroach;
 
 	b2CastOutput output = b2ShapeCast( &pairInput );
 	return output;
@@ -882,12 +912,117 @@ b2CastOutput b2ShapeCastPolygon( const b2ShapeCastInput* input, const b2Polygon*
 {
 	b2ShapeCastPairInput pairInput;
 	pairInput.proxyA = b2MakeProxy( shape->vertices, shape->count, shape->radius );
-	pairInput.proxyB = b2MakeProxy( input->points, input->count, input->radius );
+	pairInput.proxyB = input->proxy;
 	pairInput.transformA = b2Transform_identity;
 	pairInput.transformB = b2Transform_identity;
 	pairInput.translationB = input->translation;
 	pairInput.maxFraction = input->maxFraction;
+	pairInput.canEncroach = input->canEncroach;
 
 	b2CastOutput output = b2ShapeCast( &pairInput );
 	return output;
 }
+
+b2PlaneResult b2CollideMoverAndCircle( const b2Circle* shape, const b2Capsule* mover )
+{
+	b2DistanceInput distanceInput;
+	distanceInput.proxyA = b2MakeProxy( &shape->center, 1, 0.0f );
+	distanceInput.proxyB = b2MakeProxy( &mover->center1, 2, mover->radius );
+	distanceInput.transformA = b2Transform_identity;
+	distanceInput.transformB = b2Transform_identity;
+	distanceInput.useRadii = false;
+
+	float totalRadius = mover->radius + shape->radius;
+
+	b2SimplexCache cache = { 0 };
+	b2DistanceOutput distanceOutput = b2ShapeDistance( &distanceInput, &cache, NULL, 0 );
+
+	if ( distanceOutput.distance <= totalRadius )
+	{
+		b2Plane plane = { distanceOutput.normal, totalRadius - distanceOutput.distance };
+		return (b2PlaneResult){
+			.plane = plane,
+			.hit = true,
+		};
+	}
+
+	return (b2PlaneResult){ 0 };
+}
+
+b2PlaneResult b2CollideMoverAndCapsule( const b2Capsule* shape, const b2Capsule* mover )
+{
+	b2DistanceInput distanceInput;
+	distanceInput.proxyA = b2MakeProxy( &shape->center1, 2, 0.0f );
+	distanceInput.proxyB = b2MakeProxy( &mover->center1, 2, mover->radius );
+	distanceInput.transformA = b2Transform_identity;
+	distanceInput.transformB = b2Transform_identity;
+	distanceInput.useRadii = false;
+
+	float totalRadius = mover->radius + shape->radius;
+
+	b2SimplexCache cache = { 0 };
+	b2DistanceOutput distanceOutput = b2ShapeDistance( &distanceInput, &cache, NULL, 0 );
+
+	if ( distanceOutput.distance <= totalRadius )
+	{
+		b2Plane plane = { distanceOutput.normal, totalRadius - distanceOutput.distance };
+		return (b2PlaneResult){
+			.plane = plane,
+			.hit = true,
+		};
+	}
+
+	return (b2PlaneResult){ 0 };
+}
+
+b2PlaneResult b2CollideMoverAndPolygon( const b2Polygon* shape, const b2Capsule* mover )
+{
+	b2DistanceInput distanceInput;
+	distanceInput.proxyA = b2MakeProxy( shape->vertices, shape->count, shape->radius );
+	distanceInput.proxyB = b2MakeProxy( &mover->center1, 2, mover->radius );
+	distanceInput.transformA = b2Transform_identity;
+	distanceInput.transformB = b2Transform_identity;
+	distanceInput.useRadii = false;
+
+	float totalRadius = mover->radius + shape->radius;
+
+	b2SimplexCache cache = { 0 };
+	b2DistanceOutput distanceOutput = b2ShapeDistance( &distanceInput, &cache, NULL, 0 );
+
+	if ( distanceOutput.distance <= totalRadius )
+	{
+		b2Plane plane = { distanceOutput.normal, totalRadius - distanceOutput.distance };
+		return (b2PlaneResult){
+			.plane = plane,
+			.hit = true,
+		};
+	}
+
+	return (b2PlaneResult){ 0 };
+}
+
+b2PlaneResult b2CollideMoverAndSegment( const b2Segment* shape, const b2Capsule* mover )
+{
+	b2DistanceInput distanceInput;
+	distanceInput.proxyA = b2MakeProxy( &shape->point1, 2, 0.0f );
+	distanceInput.proxyB = b2MakeProxy( &mover->center1, 2, mover->radius );
+	distanceInput.transformA = b2Transform_identity;
+	distanceInput.transformB = b2Transform_identity;
+	distanceInput.useRadii = false;
+
+	float totalRadius = mover->radius;
+
+	b2SimplexCache cache = { 0 };
+	b2DistanceOutput distanceOutput = b2ShapeDistance( &distanceInput, &cache, NULL, 0 );
+
+	if ( distanceOutput.distance <= totalRadius )
+	{
+		b2Plane plane = { distanceOutput.normal, totalRadius - distanceOutput.distance };
+		return (b2PlaneResult){
+			.plane = plane,
+			.hit = true,
+		};
+	}
+
+	return (b2PlaneResult){ 0 };
+}
