@@ -6,6 +6,7 @@
 #include "body.h"
 #include "contact.h"
 #include "core.h"
+#include "island.h"
 #include "shape.h"
 #include "solver.h"
 #include "solver_set.h"
@@ -15,12 +16,16 @@
 #include "box2d/box2d.h"
 
 #include <stddef.h>
+#include <string.h>
+
+B2_ARRAY_SOURCE( b2Joint, b2Joint )
+B2_ARRAY_SOURCE( b2JointSim, b2JointSim )
 
 b2DistanceJointDef b2DefaultDistanceJointDef( void )
 {
 	b2DistanceJointDef def = { 0 };
 	def.length = 1.0f;
-	def.maxLength = b2_huge;
+	def.maxLength = B2_HUGE;
 	def.internalValue = B2_SECRET_COOKIE;
 	return def;
 }
@@ -45,10 +50,17 @@ b2MouseJointDef b2DefaultMouseJointDef( void )
 	return def;
 }
 
+b2FilterJointDef b2DefaultFilterJointDef( void )
+{
+	b2FilterJointDef def = { 0 };
+	def.internalValue = B2_SECRET_COOKIE;
+	return def;
+}
+
 b2PrismaticJointDef b2DefaultPrismaticJointDef( void )
 {
 	b2PrismaticJointDef def = { 0 };
-	def.localAxisA = ( b2Vec2 ){ 1.0f, 0.0f };
+	def.localAxisA = (b2Vec2){ 1.0f, 0.0f };
 	def.internalValue = B2_SECRET_COOKIE;
 	return def;
 }
@@ -79,43 +91,37 @@ b2WheelJointDef b2DefaultWheelJointDef( void )
 	return def;
 }
 
-static b2Joint* b2GetJointFullId( b2World* world, b2JointId jointId )
+b2ExplosionDef b2DefaultExplosionDef( void )
 {
-	int id = jointId.index1 - 1;
-	b2CheckIndex( world->jointArray, id );
-	b2Joint* joint = world->jointArray + id;
-	b2CheckIndex( world->solverSetArray, joint->setIndex );
-	B2_ASSERT( joint->revision == jointId.revision );
-	return joint;
+	b2ExplosionDef def = { 0 };
+	def.maskBits = B2_DEFAULT_MASK_BITS;
+	return def;
 }
 
-b2Joint* b2GetJoint( b2World* world, int jointId )
+b2Joint* b2GetJointFullId( b2World* world, b2JointId jointId )
 {
-	b2CheckIndex( world->jointArray, jointId );
-	b2Joint* joint = world->jointArray + jointId;
+	int id = jointId.index1 - 1;
+	b2Joint* joint = b2JointArray_Get( &world->joints, id );
+	B2_ASSERT( joint->jointId == id && joint->generation == jointId.generation );
 	return joint;
 }
 
 b2JointSim* b2GetJointSim( b2World* world, b2Joint* joint )
 {
-	b2CheckIndex( world->solverSetArray, joint->setIndex );
-
 	if ( joint->setIndex == b2_awakeSet )
 	{
-		B2_ASSERT( 0 <= joint->colorIndex && joint->colorIndex < b2_graphColorCount );
+		B2_ASSERT( 0 <= joint->colorIndex && joint->colorIndex < B2_GRAPH_COLOR_COUNT );
 		b2GraphColor* color = world->constraintGraph.colors + joint->colorIndex;
-		B2_ASSERT( 0 <= joint->localIndex && joint->localIndex < color->joints.count );
-		return color->joints.data + joint->localIndex;
+		return b2JointSimArray_Get( &color->jointSims, joint->localIndex );
 	}
 
-	b2SolverSet* set = world->solverSetArray + joint->setIndex;
-	B2_ASSERT( 0 <= joint->localIndex && joint->localIndex < set->joints.count );
-	return set->joints.data + joint->localIndex;
+	b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, joint->setIndex );
+	return b2JointSimArray_Get( &set->jointSims, joint->localIndex );
 }
 
 b2JointSim* b2GetJointSimCheckType( b2JointId jointId, b2JointType type )
 {
-	B2_MAYBE_UNUSED( type );
+	B2_UNUSED( type );
 
 	b2World* world = b2GetWorld( jointId.world0 );
 	B2_ASSERT( world->locked == false );
@@ -146,15 +152,15 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 
 	// Create joint id and joint
 	int jointId = b2AllocId( &world->jointIdPool );
-	if ( jointId == b2Array( world->jointArray ).count )
+	if ( jointId == world->joints.count )
 	{
-		b2Array_Push( world->jointArray, ( b2Joint ){ 0 } );
+		b2JointArray_Push( &world->joints, (b2Joint){ 0 } );
 	}
 
-	b2Joint* joint = world->jointArray + jointId;
+	b2Joint* joint = b2JointArray_Get( &world->joints, jointId );
 	joint->jointId = jointId;
 	joint->userData = userData;
-	joint->revision += 1;
+	joint->generation += 1;
 	joint->setIndex = B2_NULL_INDEX;
 	joint->colorIndex = B2_NULL_INDEX;
 	joint->localIndex = B2_NULL_INDEX;
@@ -174,7 +180,7 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 	int keyA = ( jointId << 1 ) | 0;
 	if ( bodyA->headJointKey != B2_NULL_INDEX )
 	{
-		b2Joint* jointA = world->jointArray + ( bodyA->headJointKey >> 1 );
+		b2Joint* jointA = b2JointArray_Get( &world->joints, bodyA->headJointKey >> 1 );
 		b2JointEdge* edgeA = jointA->edges + ( bodyA->headJointKey & 1 );
 		edgeA->prevKey = keyA;
 	}
@@ -189,7 +195,7 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 	int keyB = ( jointId << 1 ) | 1;
 	if ( bodyB->headJointKey != B2_NULL_INDEX )
 	{
-		b2Joint* jointB = world->jointArray + ( bodyB->headJointKey >> 1 );
+		b2Joint* jointB = b2JointArray_Get( &world->joints, bodyB->headJointKey >> 1 );
 		b2JointEdge* edgeB = jointB->edges + ( bodyB->headJointKey & 1 );
 		edgeB->prevKey = keyB;
 	}
@@ -201,11 +207,13 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 	if ( bodyA->setIndex == b2_disabledSet || bodyB->setIndex == b2_disabledSet )
 	{
 		// if either body is disabled, create in disabled set
-		b2SolverSet* set = world->solverSetArray + b2_disabledSet;
+		b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, b2_disabledSet );
 		joint->setIndex = b2_disabledSet;
-		joint->localIndex = set->joints.count;
+		joint->localIndex = set->jointSims.count;
+
+		jointSim = b2JointSimArray_Add( &set->jointSims );
+		memset( jointSim, 0, sizeof( b2JointSim ) );
 
-		jointSim = b2AddJoint( &set->joints );
 		jointSim->jointId = jointId;
 		jointSim->bodyIdA = bodyIdA;
 		jointSim->bodyIdB = bodyIdB;
@@ -213,11 +221,13 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 	else if ( bodyA->setIndex == b2_staticSet && bodyB->setIndex == b2_staticSet )
 	{
 		// joint is connecting static bodies
-		b2SolverSet* set = world->solverSetArray + b2_staticSet;
+		b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, b2_staticSet );
 		joint->setIndex = b2_staticSet;
-		joint->localIndex = set->joints.count;
+		joint->localIndex = set->jointSims.count;
+
+		jointSim = b2JointSimArray_Add( &set->jointSims );
+		memset( jointSim, 0, sizeof( b2JointSim ) );
 
-		jointSim = b2AddJoint( &set->joints );
 		jointSim->jointId = jointId;
 		jointSim->bodyIdA = bodyIdA;
 		jointSim->bodyIdB = bodyIdB;
@@ -246,11 +256,13 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 		// joint should go into the sleeping set (not static set)
 		int setIndex = maxSetIndex;
 
-		b2CheckIndex( world->solverSetArray, setIndex );
-		b2SolverSet* set = world->solverSetArray + setIndex;
+		b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, setIndex );
 		joint->setIndex = setIndex;
-		joint->localIndex = set->joints.count;
-		jointSim = b2AddJoint( &set->joints );
+		joint->localIndex = set->jointSims.count;
+
+		jointSim = b2JointSimArray_Add( &set->jointSims );
+		memset( jointSim, 0, sizeof( b2JointSim ) );
+
 		jointSim->jointId = jointId;
 		jointSim->bodyIdA = bodyIdA;
 		jointSim->bodyIdB = bodyIdB;
@@ -265,8 +277,10 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 			// fix potentially invalid set index
 			setIndex = bodyA->setIndex;
 
+			b2SolverSet* mergedSet = b2SolverSetArray_Get( &world->solverSets, setIndex );
+
 			// Careful! The joint sim pointer was orphaned by the set merge.
-			jointSim = world->solverSetArray[setIndex].joints.data + joint->localIndex;
+			jointSim = b2JointSimArray_Get( &mergedSet->jointSims, joint->localIndex );
 		}
 
 		B2_ASSERT( joint->setIndex == setIndex );
@@ -279,12 +293,13 @@ static b2JointPair b2CreateJoint( b2World* world, b2Body* bodyA, b2Body* bodyB,
 	if ( joint->setIndex > b2_disabledSet )
 	{
 		// Add edge to island graph
-		b2LinkJoint( world, joint );
+		bool mergeIslands = true;
+		b2LinkJoint( world, joint, mergeIslands );
 	}
 
 	b2ValidateSolverSets( world );
 
-	return ( b2JointPair ){ joint, jointSim };
+	return (b2JointPair){ joint, jointSim };
 }
 
 static void b2DestroyContactsBetweenBodies( b2World* world, b2Body* bodyA, b2Body* bodyB )
@@ -313,8 +328,7 @@ static void b2DestroyContactsBetweenBodies( b2World* world, b2Body* bodyA, b2Bod
 		int contactId = contactKey >> 1;
 		int edgeIndex = contactKey & 1;
 
-		b2CheckIndex( world->contactArray, contactId );
-		b2Contact* contact = world->contactArray + contactId;
+		b2Contact* contact = b2ContactArray_Get( &world->contacts, contactId );
 		contactKey = contact->edges[edgeIndex].nextKey;
 
 		int otherEdgeIndex = edgeIndex ^ 1;
@@ -330,19 +344,19 @@ static void b2DestroyContactsBetweenBodies( b2World* world, b2Body* bodyA, b2Bod
 
 b2JointId b2CreateDistanceJoint( b2WorldId worldId, const b2DistanceJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
 	}
 
 	B2_ASSERT( b2Body_IsValid( def->bodyIdA ) );
 	B2_ASSERT( b2Body_IsValid( def->bodyIdB ) );
-	B2_ASSERT( b2IsValid( def->length ) && def->length > 0.0f );
+	B2_ASSERT( b2IsValidFloat( def->length ) && def->length > 0.0f );
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
 	b2Body* bodyB = b2GetBodyFullId( world, def->bodyIdB );
@@ -356,10 +370,10 @@ b2JointId b2CreateDistanceJoint( b2WorldId worldId, const b2DistanceJointDef* de
 
 	b2DistanceJoint empty = { 0 };
 	joint->distanceJoint = empty;
-	joint->distanceJoint.length = b2MaxFloat( def->length, b2_linearSlop );
+	joint->distanceJoint.length = b2MaxFloat( def->length, B2_LINEAR_SLOP );
 	joint->distanceJoint.hertz = def->hertz;
 	joint->distanceJoint.dampingRatio = def->dampingRatio;
-	joint->distanceJoint.minLength = b2MaxFloat( def->minLength, b2_linearSlop );
+	joint->distanceJoint.minLength = b2MaxFloat( def->minLength, B2_LINEAR_SLOP );
 	joint->distanceJoint.maxLength = b2MaxFloat( def->minLength, def->maxLength );
 	joint->distanceJoint.maxMotorForce = def->maxMotorForce;
 	joint->distanceJoint.motorSpeed = def->motorSpeed;
@@ -377,20 +391,20 @@ b2JointId b2CreateDistanceJoint( b2WorldId worldId, const b2DistanceJointDef* de
 		b2DestroyContactsBetweenBodies( world, bodyA, bodyB );
 	}
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
 b2JointId b2CreateMotorJoint( b2WorldId worldId, const b2MotorJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
 	}
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
@@ -400,9 +414,9 @@ b2JointId b2CreateMotorJoint( b2WorldId worldId, const b2MotorJointDef* def )
 	b2JointSim* joint = pair.jointSim;
 
 	joint->type = b2_motorJoint;
-	joint->localOriginAnchorA = ( b2Vec2 ){ 0.0f, 0.0f };
-	joint->localOriginAnchorB = ( b2Vec2 ){ 0.0f, 0.0f };
-	joint->motorJoint = ( b2MotorJoint ){ 0 };
+	joint->localOriginAnchorA = (b2Vec2){ 0.0f, 0.0f };
+	joint->localOriginAnchorB = (b2Vec2){ 0.0f, 0.0f };
+	joint->motorJoint = (b2MotorJoint){ 0 };
 	joint->motorJoint.linearOffset = def->linearOffset;
 	joint->motorJoint.angularOffset = def->angularOffset;
 	joint->motorJoint.maxForce = def->maxForce;
@@ -415,20 +429,20 @@ b2JointId b2CreateMotorJoint( b2WorldId worldId, const b2MotorJointDef* def )
 		b2DestroyContactsBetweenBodies( world, bodyA, bodyB );
 	}
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
 b2JointId b2CreateMouseJoint( b2WorldId worldId, const b2MouseJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
 	}
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
@@ -451,20 +465,51 @@ b2JointId b2CreateMouseJoint( b2WorldId worldId, const b2MouseJointDef* def )
 	joint->mouseJoint.dampingRatio = def->dampingRatio;
 	joint->mouseJoint.maxForce = def->maxForce;
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
-b2JointId b2CreateRevoluteJoint( b2WorldId worldId, const b2RevoluteJointDef* def )
+b2JointId b2CreateFilterJoint( b2WorldId worldId, const b2FilterJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
+	}
+
+	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
+	b2Body* bodyB = b2GetBodyFullId( world, def->bodyIdB );
+
+	bool collideConnected = false;
+	b2JointPair pair = b2CreateJoint( world, bodyA, bodyB, def->userData, 1.0f, b2_filterJoint, collideConnected );
+
+	b2JointSim* joint = pair.jointSim;
+	joint->type = b2_filterJoint;
+	joint->localOriginAnchorA = b2Vec2_zero;
+	joint->localOriginAnchorB = b2Vec2_zero;
+
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
+	return jointId;
+}
+
+b2JointId b2CreateRevoluteJoint( b2WorldId worldId, const b2RevoluteJointDef* def )
+{
+	B2_CHECK_DEF( def );
+	B2_ASSERT( def->lowerAngle <= def->upperAngle );
+	B2_ASSERT( def->lowerAngle >= -0.95f * B2_PI );
+	B2_ASSERT( def->upperAngle <= 0.95f * B2_PI );
+
+	b2World* world = b2GetWorldFromId( worldId );
+
+	B2_ASSERT( world->locked == false );
+
+	if ( world->locked )
+	{
+		return (b2JointId){ 0 };
 	}
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
@@ -481,7 +526,7 @@ b2JointId b2CreateRevoluteJoint( b2WorldId worldId, const b2RevoluteJointDef* de
 	b2RevoluteJoint empty = { 0 };
 	joint->revoluteJoint = empty;
 
-	joint->revoluteJoint.referenceAngle = b2ClampFloat( def->referenceAngle, -b2_pi, b2_pi );
+	joint->revoluteJoint.referenceAngle = b2ClampFloat( def->referenceAngle, -B2_PI, B2_PI );
 	joint->revoluteJoint.linearImpulse = b2Vec2_zero;
 	joint->revoluteJoint.axialMass = 0.0f;
 	joint->revoluteJoint.springImpulse = 0.0f;
@@ -490,10 +535,8 @@ b2JointId b2CreateRevoluteJoint( b2WorldId worldId, const b2RevoluteJointDef* de
 	joint->revoluteJoint.upperImpulse = 0.0f;
 	joint->revoluteJoint.hertz = def->hertz;
 	joint->revoluteJoint.dampingRatio = def->dampingRatio;
-	joint->revoluteJoint.lowerAngle = b2MinFloat( def->lowerAngle, def->upperAngle );
-	joint->revoluteJoint.upperAngle = b2MaxFloat( def->lowerAngle, def->upperAngle );
-	joint->revoluteJoint.lowerAngle = b2ClampFloat( joint->revoluteJoint.lowerAngle, -b2_pi, b2_pi );
-	joint->revoluteJoint.upperAngle = b2ClampFloat( joint->revoluteJoint.upperAngle, -b2_pi, b2_pi );
+	joint->revoluteJoint.lowerAngle = def->lowerAngle;
+	joint->revoluteJoint.upperAngle = def->upperAngle;
 	joint->revoluteJoint.maxMotorTorque = def->maxMotorTorque;
 	joint->revoluteJoint.motorSpeed = def->motorSpeed;
 	joint->revoluteJoint.enableSpring = def->enableSpring;
@@ -506,20 +549,22 @@ b2JointId b2CreateRevoluteJoint( b2WorldId worldId, const b2RevoluteJointDef* de
 		b2DestroyContactsBetweenBodies( world, bodyA, bodyB );
 	}
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
 b2JointId b2CreatePrismaticJoint( b2WorldId worldId, const b2PrismaticJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
+	B2_ASSERT( def->lowerTranslation <= def->upperTranslation );
+
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
 	}
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
@@ -559,20 +604,20 @@ b2JointId b2CreatePrismaticJoint( b2WorldId worldId, const b2PrismaticJointDef*
 		b2DestroyContactsBetweenBodies( world, bodyA, bodyB );
 	}
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
 b2JointId b2CreateWeldJoint( b2WorldId worldId, const b2WeldJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
 	}
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
@@ -601,20 +646,22 @@ b2JointId b2CreateWeldJoint( b2WorldId worldId, const b2WeldJointDef* def )
 		b2DestroyContactsBetweenBodies( world, bodyA, bodyB );
 	}
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
 b2JointId b2CreateWheelJoint( b2WorldId worldId, const b2WheelJointDef* def )
 {
-	b2CheckDef( def );
+	B2_CHECK_DEF( def );
+	B2_ASSERT( def->lowerTranslation <= def->upperTranslation );
+
 	b2World* world = b2GetWorldFromId( worldId );
 
 	B2_ASSERT( world->locked == false );
 
 	if ( world->locked )
 	{
-		return ( b2JointId ){ 0 };
+		return (b2JointId){ 0 };
 	}
 
 	b2Body* bodyA = b2GetBodyFullId( world, def->bodyIdA );
@@ -627,7 +674,7 @@ b2JointId b2CreateWheelJoint( b2WorldId worldId, const b2WheelJointDef* def )
 	joint->localOriginAnchorA = def->localAnchorA;
 	joint->localOriginAnchorB = def->localAnchorB;
 
-	joint->wheelJoint = ( b2WheelJoint ){ 0 };
+	joint->wheelJoint = (b2WheelJoint){ 0 };
 	joint->wheelJoint.localAxisA = b2Normalize( def->localAxisA );
 	joint->wheelJoint.perpMass = 0.0f;
 	joint->wheelJoint.axialMass = 0.0f;
@@ -650,7 +697,7 @@ b2JointId b2CreateWheelJoint( b2WorldId worldId, const b2WheelJointDef* def )
 		b2DestroyContactsBetweenBodies( world, bodyA, bodyB );
 	}
 
-	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->revision };
+	b2JointId jointId = { joint->jointId + 1, world->worldId, pair.joint->generation };
 	return jointId;
 }
 
@@ -663,20 +710,20 @@ void b2DestroyJointInternal( b2World* world, b2Joint* joint, bool wakeBodies )
 
 	int idA = edgeA->bodyId;
 	int idB = edgeB->bodyId;
-	b2Body* bodyA = b2GetBody( world, idA );
-	b2Body* bodyB = b2GetBody( world, idB );
+	b2Body* bodyA = b2BodyArray_Get( &world->bodies, idA );
+	b2Body* bodyB = b2BodyArray_Get( &world->bodies, idB );
 
 	// Remove from body A
 	if ( edgeA->prevKey != B2_NULL_INDEX )
 	{
-		b2Joint* prevJoint = world->jointArray + ( edgeA->prevKey >> 1 );
+		b2Joint* prevJoint = b2JointArray_Get( &world->joints, edgeA->prevKey >> 1 );
 		b2JointEdge* prevEdge = prevJoint->edges + ( edgeA->prevKey & 1 );
 		prevEdge->nextKey = edgeA->nextKey;
 	}
 
 	if ( edgeA->nextKey != B2_NULL_INDEX )
 	{
-		b2Joint* nextJoint = world->jointArray + ( edgeA->nextKey >> 1 );
+		b2Joint* nextJoint = b2JointArray_Get( &world->joints, edgeA->nextKey >> 1 );
 		b2JointEdge* nextEdge = nextJoint->edges + ( edgeA->nextKey & 1 );
 		nextEdge->prevKey = edgeA->prevKey;
 	}
@@ -692,14 +739,14 @@ void b2DestroyJointInternal( b2World* world, b2Joint* joint, bool wakeBodies )
 	// Remove from body B
 	if ( edgeB->prevKey != B2_NULL_INDEX )
 	{
-		b2Joint* prevJoint = world->jointArray + ( edgeB->prevKey >> 1 );
+		b2Joint* prevJoint = b2JointArray_Get( &world->joints, edgeB->prevKey >> 1 );
 		b2JointEdge* prevEdge = prevJoint->edges + ( edgeB->prevKey & 1 );
 		prevEdge->nextKey = edgeB->nextKey;
 	}
 
 	if ( edgeB->nextKey != B2_NULL_INDEX )
 	{
-		b2Joint* nextJoint = world->jointArray + ( edgeB->nextKey >> 1 );
+		b2Joint* nextJoint = b2JointArray_Get( &world->joints, edgeB->nextKey >> 1 );
 		b2JointEdge* nextEdge = nextJoint->edges + ( edgeB->nextKey & 1 );
 		nextEdge->prevKey = edgeB->prevKey;
 	}
@@ -712,7 +759,15 @@ void b2DestroyJointInternal( b2World* world, b2Joint* joint, bool wakeBodies )
 
 	bodyB->jointCount -= 1;
 
-	b2UnlinkJoint( world, joint );
+	if ( joint->islandId != B2_NULL_INDEX )
+	{
+		B2_ASSERT( joint->setIndex > b2_disabledSet );
+		b2UnlinkJoint( world, joint );
+	}
+	else
+	{
+		B2_ASSERT( joint->setIndex <= b2_disabledSet );
+	}
 
 	// Remove joint from solver set that owns it
 	int setIndex = joint->setIndex;
@@ -724,20 +779,20 @@ void b2DestroyJointInternal( b2World* world, b2Joint* joint, bool wakeBodies )
 	}
 	else
 	{
-		b2SolverSet* set = world->solverSetArray + setIndex;
-		int movedIndex = b2RemoveJoint( &set->joints, localIndex );
+		b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, setIndex );
+		int movedIndex = b2JointSimArray_RemoveSwap( &set->jointSims, localIndex );
 		if ( movedIndex != B2_NULL_INDEX )
 		{
 			// Fix moved joint
-			b2JointSim* movedJointSim = set->joints.data + localIndex;
+			b2JointSim* movedJointSim = set->jointSims.data + localIndex;
 			int movedId = movedJointSim->jointId;
-			b2Joint* movedJoint = world->jointArray + movedId;
+			b2Joint* movedJoint = b2JointArray_Get( &world->joints, movedId );
 			B2_ASSERT( movedJoint->localIndex == movedIndex );
 			movedJoint->localIndex = localIndex;
 		}
 	}
 
-	// Free joint and id (preserve joint revision)
+	// Free joint and id (preserve joint generation)
 	joint->setIndex = B2_NULL_INDEX;
 	joint->localIndex = B2_NULL_INDEX;
 	joint->colorIndex = B2_NULL_INDEX;
@@ -789,6 +844,12 @@ b2BodyId b2Joint_GetBodyB( b2JointId jointId )
 	return b2MakeBodyId( world, joint->edges[1].bodyId );
 }
 
+b2WorldId b2Joint_GetWorld( b2JointId jointId )
+{
+	b2World* world = b2GetWorld( jointId.world0 );
+	return (b2WorldId){ jointId.world0 + 1, world->generation };
+}
+
 b2Vec2 b2Joint_GetLocalAnchorA( b2JointId jointId )
 {
 	b2World* world = b2GetWorld( jointId.world0 );
@@ -821,8 +882,8 @@ void b2Joint_SetCollideConnected( b2JointId jointId, bool shouldCollide )
 
 	joint->collideConnected = shouldCollide;
 
-	b2Body* bodyA = b2GetBody( world, joint->edges[0].bodyId );
-	b2Body* bodyB = b2GetBody( world, joint->edges[1].bodyId );
+	b2Body* bodyA = b2BodyArray_Get( &world->bodies, joint->edges[0].bodyId );
+	b2Body* bodyB = b2BodyArray_Get( &world->bodies, joint->edges[1].bodyId );
 
 	if ( shouldCollide )
 	{
@@ -834,7 +895,7 @@ void b2Joint_SetCollideConnected( b2JointId jointId, bool shouldCollide )
 		int shapeId = shapeCountA < shapeCountB ? bodyA->headShapeId : bodyB->headShapeId;
 		while ( shapeId != B2_NULL_INDEX )
 		{
-			b2Shape* shape = world->shapeArray + shapeId;
+			b2Shape* shape = b2ShapeArray_Get( &world->shapes, shapeId );
 
 			if ( shape->proxyKey != B2_NULL_INDEX )
 			{
@@ -880,21 +941,13 @@ void b2Joint_WakeBodies( b2JointId jointId )
 	}
 
 	b2Joint* joint = b2GetJointFullId( world, jointId );
-	b2Body* bodyA = world->bodyArray + joint->edges[0].bodyId;
-	b2Body* bodyB = world->bodyArray + joint->edges[1].bodyId;
+	b2Body* bodyA = b2BodyArray_Get( &world->bodies, joint->edges[0].bodyId );
+	b2Body* bodyB = b2BodyArray_Get( &world->bodies, joint->edges[1].bodyId );
 
 	b2WakeBody( world, bodyA );
 	b2WakeBody( world, bodyB );
 }
 
-extern b2Vec2 b2GetDistanceJointForce( b2World* world, b2JointSim* base );
-extern b2Vec2 b2GetMotorJointForce( b2World* world, b2JointSim* base );
-extern b2Vec2 b2GetMouseJointForce( b2World* world, b2JointSim* base );
-extern b2Vec2 b2GetPrismaticJointForce( b2World* world, b2JointSim* base );
-extern b2Vec2 b2GetRevoluteJointForce( b2World* world, b2JointSim* base );
-extern b2Vec2 b2GetWeldJointForce( b2World* world, b2JointSim* base );
-extern b2Vec2 b2GetWheelJointForce( b2World* world, b2JointSim* base );
-
 b2Vec2 b2Joint_GetConstraintForce( b2JointId jointId )
 {
 	b2World* world = b2GetWorld( jointId.world0 );
@@ -912,6 +965,9 @@ b2Vec2 b2Joint_GetConstraintForce( b2JointId jointId )
 		case b2_mouseJoint:
 			return b2GetMouseJointForce( world, base );
 
+		case b2_filterJoint:
+			return b2Vec2_zero;
+
 		case b2_prismaticJoint:
 			return b2GetPrismaticJointForce( world, base );
 
@@ -930,13 +986,6 @@ b2Vec2 b2Joint_GetConstraintForce( b2JointId jointId )
 	}
 }
 
-extern float b2GetMotorJointTorque( b2World* world, b2JointSim* base );
-extern float b2GetMouseJointTorque( b2World* world, b2JointSim* base );
-extern float b2GetPrismaticJointTorque( b2World* world, b2JointSim* base );
-extern float b2GetRevoluteJointTorque( b2World* world, b2JointSim* base );
-extern float b2GetWeldJointTorque( b2World* world, b2JointSim* base );
-extern float b2GetWheelJointTorque( b2World* world, b2JointSim* base );
-
 float b2Joint_GetConstraintTorque( b2JointId jointId )
 {
 	b2World* world = b2GetWorld( jointId.world0 );
@@ -954,6 +1003,9 @@ float b2Joint_GetConstraintTorque( b2JointId jointId )
 		case b2_mouseJoint:
 			return b2GetMouseJointTorque( world, base );
 
+		case b2_filterJoint:
+			return 0.0f;
+
 		case b2_prismaticJoint:
 			return b2GetPrismaticJointTorque( world, base );
 
@@ -972,14 +1024,6 @@ float b2Joint_GetConstraintTorque( b2JointId jointId )
 	}
 }
 
-extern void b2PrepareDistanceJoint( b2JointSim* base, b2StepContext* context );
-extern void b2PrepareMotorJoint( b2JointSim* base, b2StepContext* context );
-extern void b2PrepareMouseJoint( b2JointSim* base, b2StepContext* context );
-extern void b2PreparePrismaticJoint( b2JointSim* base, b2StepContext* context );
-extern void b2PrepareRevoluteJoint( b2JointSim* base, b2StepContext* context );
-extern void b2PrepareWeldJoint( b2JointSim* base, b2StepContext* context );
-extern void b2PrepareWheelJoint( b2JointSim* base, b2StepContext* context );
-
 void b2PrepareJoint( b2JointSim* joint, b2StepContext* context )
 {
 	switch ( joint->type )
@@ -996,6 +1040,9 @@ void b2PrepareJoint( b2JointSim* joint, b2StepContext* context )
 			b2PrepareMouseJoint( joint, context );
 			break;
 
+		case b2_filterJoint:
+			break;
+
 		case b2_prismaticJoint:
 			b2PreparePrismaticJoint( joint, context );
 			break;
@@ -1017,14 +1064,6 @@ void b2PrepareJoint( b2JointSim* joint, b2StepContext* context )
 	}
 }
 
-extern void b2WarmStartDistanceJoint( b2JointSim* base, b2StepContext* context );
-extern void b2WarmStartMotorJoint( b2JointSim* base, b2StepContext* context );
-extern void b2WarmStartMouseJoint( b2JointSim* base, b2StepContext* context );
-extern void b2WarmStartPrismaticJoint( b2JointSim* base, b2StepContext* context );
-extern void b2WarmStartRevoluteJoint( b2JointSim* base, b2StepContext* context );
-extern void b2WarmStartWeldJoint( b2JointSim* base, b2StepContext* context );
-extern void b2WarmStartWheelJoint( b2JointSim* base, b2StepContext* context );
-
 void b2WarmStartJoint( b2JointSim* joint, b2StepContext* context )
 {
 	switch ( joint->type )
@@ -1041,6 +1080,9 @@ void b2WarmStartJoint( b2JointSim* joint, b2StepContext* context )
 			b2WarmStartMouseJoint( joint, context );
 			break;
 
+		case b2_filterJoint:
+			break;
+
 		case b2_prismaticJoint:
 			b2WarmStartPrismaticJoint( joint, context );
 			break;
@@ -1062,14 +1104,6 @@ void b2WarmStartJoint( b2JointSim* joint, b2StepContext* context )
 	}
 }
 
-extern void b2SolveDistanceJoint( b2JointSim* base, b2StepContext* context, bool useBias );
-extern void b2SolveMotorJoint( b2JointSim* base, b2StepContext* context, bool useBias );
-extern void b2SolveMouseJoint( b2JointSim* base, b2StepContext* context );
-extern void b2SolvePrismaticJoint( b2JointSim* base, b2StepContext* context, bool useBias );
-extern void b2SolveRevoluteJoint( b2JointSim* base, b2StepContext* context, bool useBias );
-extern void b2SolveWeldJoint( b2JointSim* base, b2StepContext* context, bool useBias );
-extern void b2SolveWheelJoint( b2JointSim* base, b2StepContext* context, bool useBias );
-
 void b2SolveJoint( b2JointSim* joint, b2StepContext* context, bool useBias )
 {
 	switch ( joint->type )
@@ -1086,6 +1120,9 @@ void b2SolveJoint( b2JointSim* joint, b2StepContext* context, bool useBias )
 			b2SolveMouseJoint( joint, context );
 			break;
 
+		case b2_filterJoint:
+			break;
+
 		case b2_prismaticJoint:
 			b2SolvePrismaticJoint( joint, context, useBias );
 			break;
@@ -1111,10 +1148,9 @@ void b2PrepareOverflowJoints( b2StepContext* context )
 {
 	b2TracyCZoneNC( prepare_joints, "PrepJoints", b2_colorOldLace, true );
 
-	b2World* world = context->world;
 	b2ConstraintGraph* graph = context->graph;
-	b2JointSim* joints = graph->colors[b2_overflowIndex].joints.data;
-	int jointCount = graph->colors[b2_overflowIndex].joints.count;
+	b2JointSim* joints = graph->colors[B2_OVERFLOW_INDEX].jointSims.data;
+	int jointCount = graph->colors[B2_OVERFLOW_INDEX].jointSims.count;
 
 	for ( int i = 0; i < jointCount; ++i )
 	{
@@ -1129,10 +1165,9 @@ void b2WarmStartOverflowJoints( b2StepContext* context )
 {
 	b2TracyCZoneNC( prepare_joints, "PrepJoints", b2_colorOldLace, true );
 
-	b2World* world = context->world;
 	b2ConstraintGraph* graph = context->graph;
-	b2JointSim* joints = graph->colors[b2_overflowIndex].joints.data;
-	int jointCount = graph->colors[b2_overflowIndex].joints.count;
+	b2JointSim* joints = graph->colors[B2_OVERFLOW_INDEX].jointSims.data;
+	int jointCount = graph->colors[B2_OVERFLOW_INDEX].jointSims.count;
 
 	for ( int i = 0; i < jointCount; ++i )
 	{
@@ -1147,10 +1182,9 @@ void b2SolveOverflowJoints( b2StepContext* context, bool useBias )
 {
 	b2TracyCZoneNC( solve_joints, "SolveJoints", b2_colorLemonChiffon, true );
 
-	b2World* world = context->world;
 	b2ConstraintGraph* graph = context->graph;
-	b2JointSim* joints = graph->colors[b2_overflowIndex].joints.data;
-	int jointCount = graph->colors[b2_overflowIndex].joints.count;
+	b2JointSim* joints = graph->colors[B2_OVERFLOW_INDEX].jointSims.data;
+	int jointCount = graph->colors[B2_OVERFLOW_INDEX].jointSims.count;
 
 	for ( int i = 0; i < jointCount; ++i )
 	{
@@ -1161,16 +1195,10 @@ void b2SolveOverflowJoints( b2StepContext* context, bool useBias )
 	b2TracyCZoneEnd( solve_joints );
 }
 
-extern void b2DrawDistanceJoint( b2DebugDraw* draw, b2JointSim* base, b2Transform transformA, b2Transform transformB );
-extern void b2DrawPrismaticJoint( b2DebugDraw* draw, b2JointSim* base, b2Transform transformA, b2Transform transformB );
-extern void b2DrawRevoluteJoint( b2DebugDraw* draw, b2JointSim* base, b2Transform transformA, b2Transform transformB,
-								 float drawSize );
-extern void b2DrawWheelJoint( b2DebugDraw* draw, b2JointSim* base, b2Transform transformA, b2Transform transformB );
-
 void b2DrawJoint( b2DebugDraw* draw, b2World* world, b2Joint* joint )
 {
-	b2Body* bodyA = b2GetBody( world, joint->edges[0].bodyId );
-	b2Body* bodyB = b2GetBody( world, joint->edges[1].bodyId );
+	b2Body* bodyA = b2BodyArray_Get( &world->bodies, joint->edges[0].bodyId );
+	b2Body* bodyB = b2BodyArray_Get( &world->bodies, joint->edges[1].bodyId );
 	if ( bodyA->setIndex == b2_disabledSet || bodyB->setIndex == b2_disabledSet )
 	{
 		return;
@@ -1196,11 +1224,17 @@ void b2DrawJoint( b2DebugDraw* draw, b2World* world, b2Joint* joint )
 			b2Vec2 target = jointSim->mouseJoint.targetA;
 
 			b2HexColor c1 = b2_colorGreen;
-			draw->DrawPoint( target, 4.0f, c1, draw->context );
-			draw->DrawPoint( pB, 4.0f, c1, draw->context );
+			draw->DrawPointFcn( target, 4.0f, c1, draw->context );
+			draw->DrawPointFcn( pB, 4.0f, c1, draw->context );
 
-			b2HexColor c2 = b2_colorGray8;
-			draw->DrawSegment( target, pB, c2, draw->context );
+			b2HexColor c2 = b2_colorLightGray;
+			draw->DrawSegmentFcn( target, pB, c2, draw->context );
+		}
+		break;
+
+		case b2_filterJoint:
+		{
+			draw->DrawSegmentFcn( pA, pB, b2_colorGold, draw->context );
 		}
 		break;
 
@@ -1217,22 +1251,22 @@ void b2DrawJoint( b2DebugDraw* draw, b2World* world, b2Joint* joint )
 			break;
 
 		default:
-			draw->DrawSegment( transformA.p, pA, color, draw->context );
-			draw->DrawSegment( pA, pB, color, draw->context );
-			draw->DrawSegment( transformB.p, pB, color, draw->context );
+			draw->DrawSegmentFcn( transformA.p, pA, color, draw->context );
+			draw->DrawSegmentFcn( pA, pB, color, draw->context );
+			draw->DrawSegmentFcn( transformB.p, pB, color, draw->context );
 	}
 
 	if ( draw->drawGraphColors )
 	{
-		b2HexColor colors[b2_graphColorCount] = { b2_colorRed,		 b2_colorOrange,	b2_colorYellow, b2_colorGreen,
-												  b2_colorCyan,		 b2_colorBlue,		b2_colorViolet, b2_colorPink,
-												  b2_colorChocolate, b2_colorGoldenrod, b2_colorCoral,	b2_colorBlack };
+		b2HexColor colors[B2_GRAPH_COLOR_COUNT] = { b2_colorRed,	   b2_colorOrange,	  b2_colorYellow, b2_colorGreen,
+													b2_colorCyan,	   b2_colorBlue,	  b2_colorViolet, b2_colorPink,
+													b2_colorChocolate, b2_colorGoldenRod, b2_colorCoral,  b2_colorBlack };
 
 		int colorIndex = joint->colorIndex;
 		if ( colorIndex != B2_NULL_INDEX )
 		{
 			b2Vec2 p = b2Lerp( pA, pB, 0.5f );
-			draw->DrawPoint( p, 5.0f, colors[colorIndex], draw->context );
+			draw->DrawPointFcn( p, 5.0f, colors[colorIndex], draw->context );
 		}
 	}
 }
